"""Test that boolean conditions can be simplified"""
# pylint: disable=pointless-statement


def func(_):
    """Pointless function"""


CONSTANT = 100
OTHER = 200

# Simplifies any boolean expression that is coerced into a True/False value
bool(CONSTANT or False)  # [simplifiable-condition]
assert CONSTANT or False  # [simplifiable-condition]
if CONSTANT and True:  # [simplifiable-condition]
    pass
elif CONSTANT and True:  # [simplifiable-condition]
    pass
while CONSTANT and True:  # [simplifiable-condition]
    break
1 if CONSTANT or False else 2  # [simplifiable-condition]
z = [x for x in range(10) if x or False]  # [simplifiable-condition]

# Simplifies recursively
assert CONSTANT or (True and False)  # [simplifiable-condition]
assert True and CONSTANT and OTHER  # [simplifiable-condition]
assert (CONSTANT or False) and (OTHER or True)  # [simplifiable-condition]

# Will try to infer the truthiness of an expression as long as it doesn't contain any variables
assert [] or CONSTANT  # [simplifiable-condition]
assert {} or CONSTANT  # [simplifiable-condition]

# Expressions not in one of the above situations will not emit a message
CONSTANT or True
bool(CONSTANT or OTHER)
bool(func(CONSTANT or True))



# pylint: disable=missing-docstring,too-few-public-methods
# pylint: disable=unpacking-non-sequence,attribute-defined-outside-init,invalid-name

class Class:
    pass

CLS = Class()
FIRST = 1
# Not enough values on the right hand side
FIRST, SECOND = FIRST
# Not enough values on the left hand side
FIRST = FIRST, SECOND
# Not equivalent to a self assignment
FIRST = (FIRST, )
# Not assigning to an attribute
CLS.FIRST = FIRST
# Not a name on the right hand side
FIRST = Class()
FIRST = FIRST # [self-assigning-variable]
FIRST, SECOND = FIRST, CLS.FIRST # [self-assigning-variable]


FOO = 1
FOO, = [FOO]


class RedefinedModuleLevel:
    FOO = FOO



"""Test that `continue` is catched when met inside a `finally` clause."""

# pylint: disable=missing-docstring, lost-exception, broad-except

while True:
    try:
        pass
    finally:
        continue # [continue-in-finally]

while True:
    try:
        pass
    finally:
        break

while True:
    try:
        pass
    except Exception:
        pass
    else:
        continue



# Copyright (c) 2016-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2016, 2019 Ashley Whetter <ashley@awhetter.co.uk>
# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
# Copyright (c) 2018 Jim Robertson <jrobertson98atx@gmail.com>
# Copyright (c) 2018 Adam Dangoor <adamdangoor@gmail.com>
# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
# Copyright (c) 2019 Danny Hermes <daniel.j.hermes@gmail.com>
# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Unit tests for the raised exception documentation checking in the
`DocstringChecker` in :mod:`pylint.extensions.check_docs`
"""

# pylint: disable=too-many-public-methods

import astroid

from pylint.extensions.docparams import DocstringParameterChecker
from pylint.testutils import CheckerTestCase, Message, set_config


class TestDocstringCheckerRaise(CheckerTestCase):
    """Tests for pylint_plugin.RaiseDocChecker"""

    CHECKER_CLASS = DocstringParameterChecker

    def test_ignores_no_docstring(self):
        raise_node = astroid.extract_node(
            """
        def my_func(self):
            raise RuntimeError('hi') #@
        """
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_ignores_unknown_style(self):
        node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring."""
            raise RuntimeError('hi')
        '''
        )
        raise_node = node.body[0]
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    @set_config(accept_no_raise_doc=False)
    def test_warns_unknown_style(self):
        node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring."""
            raise RuntimeError('hi')
        '''
        )
        raise_node = node.body[0]
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("RuntimeError",))
        ):
            self.checker.visit_raise(raise_node)

    def test_find_missing_sphinx_raises(self):
        node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            :raises NameError: Never
            """
            raise RuntimeError('hi')
            raise NameError('hi')
        '''
        )
        raise_node = node.body[0]
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("RuntimeError",))
        ):
            self.checker.visit_raise(raise_node)

    def test_find_missing_google_raises(self):
        node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            Raises:
                NameError: Never
            """
            raise RuntimeError('hi')
            raise NameError('hi')
        '''
        )
        raise_node = node.body[0]
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("RuntimeError",))
        ):
            self.checker.visit_raise(raise_node)

    def test_find_google_attr_raises_exact_exc(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a google docstring.

            Raises:
                re.error: Sometimes
            """
            import re
            raise re.error('hi')  #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_google_attr_raises_substr_exc(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a google docstring.

            Raises:
                re.error: Sometimes
            """
            from re import error
            raise error('hi')  #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_valid_missing_google_attr_raises(self):
        node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a google docstring.

            Raises:
                re.anothererror: Sometimes
            """
            from re import error
            raise error('hi')
        '''
        )
        raise_node = node.body[1]
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("error",))
        ):
            self.checker.visit_raise(raise_node)

    def test_find_invalid_missing_google_attr_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a google docstring.

            Raises:
                bogusmodule.error: Sometimes
            """
            from re import error
            raise error('hi') #@
        '''
        )
        # pylint allows this to pass since the comparison between Raises and
        # raise are based on the class name, not the qualified name.
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_google_raises_local_reference(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a google docstring.

            Raises:
                .LocalException: Always
            """
            from neighbor_module import LocalException
            raise LocalException('hi') #@
        '''
        )
        # pylint allows this to pass since the comparison between Raises and
        # raise are based on the class name, not the qualified name.
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    @set_config(accept_no_raise_doc=False)
    def test_google_raises_with_prefix(self):
        code_snippet = '''
        def my_func(self):
            """This is a google docstring.

            Raises:
                {prefix}re.error: Sometimes
            """
            import re
            raise re.error('hi') #@
        '''
        for prefix in ["~", "!"]:
            raise_node = astroid.extract_node(code_snippet.format(prefix=prefix))
            with self.assertNoMessages():
                self.checker.visit_raise(raise_node)

    def test_find_missing_numpy_raises(self):
        node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            Raises
            ------
            NameError
                Never
            """
            raise RuntimeError('hi')
            raise NameError('hi')
        '''
        )
        raise_node = node.body[0]
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("RuntimeError",))
        ):
            self.checker.visit_raise(raise_node)

    def test_ignore_spurious_sphinx_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            :raises RuntimeError: Always
            :except NameError: Never
            :raise OSError: Never
            :exception ValueError: Never
            """
            raise RuntimeError('Blah') #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_all_sphinx_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            :raises RuntimeError: Always
            :except NameError: Never
            :raise OSError: Never
            :exception ValueError: Never
            """
            raise RuntimeError('hi') #@
            raise NameError('hi')
            raise OSError(2, 'abort!')
            raise ValueError('foo')
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_all_google_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            Raises:
                RuntimeError: Always
                NameError: Never
            """
            raise RuntimeError('hi') #@
            raise NameError('hi')
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_all_numpy_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            Raises
            ------
            RuntimeError
                Always
            NameError
                Never
            """
            raise RuntimeError('hi') #@
            raise NameError('hi')
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_multiple_sphinx_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            :raises RuntimeError: Always
            :raises NameError, OSError, ValueError: Never
            """
            raise RuntimeError('hi')
            raise NameError('hi') #@
            raise OSError(2, 'abort!')
            raise ValueError('foo')
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_multiple_google_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            Raises:
                RuntimeError: Always
                NameError, OSError, ValueError: Never
            """
            raise RuntimeError('hi')
            raise NameError('hi') #@
            raise OSError(2, 'abort!')
            raise ValueError('foo')
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_finds_rethrown_sphinx_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            :raises NameError: Sometimes
            """
            try:
                fake_func()
            except RuntimeError:
                raise #@

            raise NameError('hi')
        '''
        )
        node = raise_node.frame()
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("RuntimeError",))
        ):
            self.checker.visit_raise(raise_node)

    def test_find_rethrown_google_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            Raises:
                NameError: Sometimes
            """
            try:
                fake_func()
            except RuntimeError:
                raise #@

            raise NameError('hi')
        '''
        )
        node = raise_node.frame()
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("RuntimeError",))
        ):
            self.checker.visit_raise(raise_node)

    def test_find_rethrown_numpy_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            Raises
            ------
            NameError
                Sometimes
            """
            try:
                fake_func()
            except RuntimeError:
                raise #@

            raise NameError('hi')
        '''
        )
        node = raise_node.frame()
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("RuntimeError",))
        ):
            self.checker.visit_raise(raise_node)

    def test_finds_rethrown_sphinx_multiple_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            :raises NameError: Sometimes
            """
            try:
                fake_func()
            except (RuntimeError, ValueError):
                raise #@

            raise NameError('hi')
        '''
        )
        node = raise_node.frame()
        with self.assertAddsMessages(
            Message(
                msg_id="missing-raises-doc",
                node=node,
                args=("RuntimeError, ValueError",),
            )
        ):
            self.checker.visit_raise(raise_node)

    def test_find_rethrown_google_multiple_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            Raises:
                NameError: Sometimes
            """
            try:
                fake_func()
            except (RuntimeError, ValueError):
                raise #@

            raise NameError('hi')
        '''
        )
        node = raise_node.frame()
        with self.assertAddsMessages(
            Message(
                msg_id="missing-raises-doc",
                node=node,
                args=("RuntimeError, ValueError",),
            )
        ):
            self.checker.visit_raise(raise_node)

    def test_find_rethrown_numpy_multiple_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            Raises
            ------
            NameError
                Sometimes
            """
            try:
                fake_func()
            except (RuntimeError, ValueError):
                raise #@

            raise NameError('hi')
        '''
        )
        node = raise_node.frame()
        with self.assertAddsMessages(
            Message(
                msg_id="missing-raises-doc",
                node=node,
                args=("RuntimeError, ValueError",),
            )
        ):
            self.checker.visit_raise(raise_node)

    def test_ignores_caught_sphinx_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            :raises NameError: Sometimes
            """
            try:
                raise RuntimeError('hi') #@
            except RuntimeError:
                pass

            raise NameError('hi')
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_ignores_caught_google_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            Raises:
                NameError: Sometimes
            """
            try:
                raise RuntimeError('hi') #@
            except RuntimeError:
                pass

            raise NameError('hi')
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_ignores_caught_numpy_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a numpy docstring.

            Raises
            ------
            NameError
                Sometimes
            """
            try:
                raise RuntimeError('hi') #@
            except RuntimeError:
                pass

            raise NameError('hi')
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_numpy_attr_raises_exact_exc(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a numpy docstring.

            Raises
            ------
            re.error
                Sometimes
            """
            import re
            raise re.error('hi')  #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_numpy_attr_raises_substr_exc(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a numpy docstring.

            Raises
            ------
            re.error
                Sometimes
            """
            from re import error
            raise error('hi')  #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_valid_missing_numpy_attr_raises(self):
        node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a numpy docstring.

            Raises
            ------
            re.anothererror
                Sometimes
            """
            from re import error
            raise error('hi')
        '''
        )
        raise_node = node.body[1]
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("error",))
        ):
            self.checker.visit_raise(raise_node)

    def test_find_invalid_missing_numpy_attr_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a numpy docstring.

            Raises
            ------
            bogusmodule.error
                Sometimes
            """
            from re import error
            raise error('hi') #@
        '''
        )
        # pylint allows this to pass since the comparison between Raises and
        # raise are based on the class name, not the qualified name.
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    @set_config(accept_no_raise_doc=False)
    def test_numpy_raises_with_prefix(self):
        code_snippet = '''
        def my_func(self):
            """This is a numpy docstring.

            Raises
            ------
            {prefix}re.error
                Sometimes
            """
            import re
            raise re.error('hi') #@
        '''
        for prefix in ["~", "!"]:
            raise_node = astroid.extract_node(code_snippet.format(prefix=prefix))
            with self.assertNoMessages():
                self.checker.visit_raise(raise_node)

    def test_find_missing_sphinx_raises_infer_from_instance(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            :raises NameError: Never
            """
            my_exception = RuntimeError('hi')
            raise my_exception #@
            raise NameError('hi')
        '''
        )
        node = raise_node.frame()
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("RuntimeError",))
        ):
            self.checker.visit_raise(raise_node)

    def test_find_missing_sphinx_raises_infer_from_function(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            :raises NameError: Never
            """
            def ex_func(val):
                return RuntimeError(val)
            raise ex_func('hi') #@
            raise NameError('hi')
        '''
        )
        node = raise_node.frame()
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("RuntimeError",))
        ):
            self.checker.visit_raise(raise_node)

    def test_find_sphinx_attr_raises_exact_exc(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a sphinx docstring.

            :raises re.error: Sometimes
            """
            import re
            raise re.error('hi')  #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_sphinx_attr_raises_substr_exc(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a sphinx docstring.

            :raises re.error: Sometimes
            """
            from re import error
            raise error('hi')  #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_find_valid_missing_sphinx_attr_raises(self):
        node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a sphinx docstring.

            :raises re.anothererror: Sometimes
            """
            from re import error
            raise error('hi')
        '''
        )
        raise_node = node.body[1]
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("error",))
        ):
            self.checker.visit_raise(raise_node)

    def test_find_invalid_missing_sphinx_attr_raises(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a sphinx docstring.

            :raises bogusmodule.error: Sometimes
            """
            from re import error
            raise error('hi') #@
        '''
        )
        # pylint allows this to pass since the comparison between Raises and
        # raise are based on the class name, not the qualified name.
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    @set_config(accept_no_raise_doc=False)
    def test_sphinx_raises_with_prefix(self):
        code_snippet = '''
        def my_func(self):
            """This is a sphinx docstring.

            :raises {prefix}re.error: Sometimes
            """
            import re
            raise re.error('hi') #@
        '''
        for prefix in ["~", "!"]:
            raise_node = astroid.extract_node(code_snippet.format(prefix=prefix))
            with self.assertNoMessages():
                self.checker.visit_raise(raise_node)

        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_ignores_raise_uninferable(self):
        raise_node = astroid.extract_node(
            '''
        from unknown import Unknown

        def my_func(self):
            """This is a docstring.

            :raises NameError: Never
            """
            raise Unknown('hi') #@
            raise NameError('hi')
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_ignores_returns_from_inner_functions(self):
        raise_node = astroid.extract_node(
            '''
        def my_func(self):
            """This is a docstring.

            :raises NameError: Never
            """
            def ex_func(val):
                def inner_func(value):
                    return OSError(value)
                return RuntimeError(val)
            raise ex_func('hi') #@
            raise NameError('hi')
        '''
        )
        node = raise_node.frame()
        with self.assertAddsMessages(
            Message(msg_id="missing-raises-doc", node=node, args=("RuntimeError",))
        ):
            # we do NOT expect a warning about the OSError in inner_func!
            self.checker.visit_raise(raise_node)

    def test_ignores_returns_use_only_names(self):
        raise_node = astroid.extract_node(
            '''
        def myfunc():
            """This is a docstring

            :raises NameError: Never
            """
            def inner_func():
                return 42

            raise inner_func() #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_ignores_returns_use_only_exception_instances(self):
        raise_node = astroid.extract_node(
            '''
        def myfunc():
            """This is a docstring

            :raises MyException: Never
            """
            class MyException(Exception):
                pass
            def inner_func():
                return MyException

            raise inner_func() #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_no_crash_when_inferring_handlers(self):
        raise_node = astroid.extract_node(
            '''
        import collections

        def test():
           """raises

           :raise U: pass
           """
           try:
              pass
           except collections.U as exc:
              raise #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_no_crash_when_cant_find_exception(self):
        raise_node = astroid.extract_node(
            '''
        import collections

        def test():
           """raises

           :raise U: pass
           """
           try:
              pass
           except U as exc:
              raise #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)

    def test_no_error_notimplemented_documented(self):
        raise_node = astroid.extract_node(
            '''
        def my_func():
            """
            Raises:
                NotImplementedError: When called.
            """
            raise NotImplementedError #@
        '''
        )
        with self.assertNoMessages():
            self.checker.visit_raise(raise_node)



"""Check invalid value returned by __bool__ """

# pylint: disable=too-few-public-methods,missing-docstring,no-self-use,import-error, useless-object-inheritance
import six

from missing import Missing


class FirstGoodBool(object):
    """__bool__ returns <type 'bool'>"""

    def __bool__(self):
        return True


class SecondGoodBool(object):
    """__bool__ returns <type 'bool'>"""

    def __bool__(self):
        return bool(0)


class BoolMetaclass(type):
    def __bool__(cls):
        return True


@six.add_metaclass(BoolMetaclass)
class ThirdGoodBool(object):
    """Bool through the metaclass."""


class FirstBadBool(object):
    """ __bool__ returns an integer """

    def __bool__(self):  # [invalid-bool-returned]
        return 1


class SecondBadBool(object):
    """ __bool__ returns str """

    def __bool__(self):  # [invalid-bool-returned]
        return "True"


class ThirdBadBool(object):
    """ __bool__ returns node which does not have 'value' in AST """

    def __bool__(self):  # [invalid-bool-returned]
        return lambda: 3


class AmbigousBool(object):
    """ Uninferable return value """
    __bool__ = lambda self: Missing


class AnotherAmbiguousBool(object):
    """Potential uninferable return value"""
    def __bool__(self):
        return bool(Missing)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

import astroid

from pylint.checkers.refactoring import LenChecker


def test_class_tree_detection():
    module = astroid.parse(
        """
class ClassWithBool(list):
    def __bool__(self):
        return True

class ClassWithoutBool(dict):
    pass

class ChildClassWithBool(ClassWithBool):
    pass

class ChildClassWithoutBool(ClassWithoutBool):
    pass
"""
    )
    with_bool, without_bool, child_with_bool, child_without_bool = module.body
    assert LenChecker().base_classes_of_node(with_bool) == [
        "ClassWithBool",
        "list",
        "object",
    ]
    assert LenChecker().base_classes_of_node(without_bool) == [
        "ClassWithoutBool",
        "dict",
        "object",
    ]
    assert LenChecker().base_classes_of_node(child_with_bool) == [
        "ChildClassWithBool",
        "ClassWithBool",
        "list",
        "object",
    ]
    assert LenChecker().base_classes_of_node(child_without_bool) == [
        "ChildClassWithoutBool",
        "ClassWithoutBool",
        "dict",
        "object",
    ]



# Copyright (c) 2014 Vlad Temian <vladtemian@gmail.com>
# Copyright (c) 2015-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2017 guillaume2 <guillaume.peillex@gmail.col>
# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Test for the JSON reporter."""

import json
from io import StringIO

from pylint import checkers
from pylint.lint import PyLinter
from pylint.reporters import JSONReporter
from pylint.reporters.ureports.nodes import EvaluationSection

expected_score_message = "Expected score message"
expected_result = [
    [
        ("column", 0),
        ("line", 1),
        ("message", "Line too long (1/2)"),
        ("message-id", "C0301"),
        ("module", "0123"),
        ("obj", ""),
        ("path", "0123"),
        ("symbol", "line-too-long"),
        ("type", "convention"),
    ]
]


def test_simple_json_output_no_score():
    report = get_linter_result(score=False)
    assert len(report) == 1
    report_result = [sorted(report[0].items(), key=lambda item: item[0])]
    assert report_result == expected_result


def get_linter_result(score):
    output = StringIO()
    reporter = JSONReporter(output)
    linter = PyLinter(reporter=reporter)
    checkers.initialize(linter)
    linter.config.persistent = 0
    linter.config.score = score
    linter.open()
    linter.set_current_module("0123")
    linter.add_message("line-too-long", line=1, args=(1, 2))
    # we call those methods because we didn't actually run the checkers
    if score:
        reporter.display_reports(EvaluationSection(expected_score_message))
    reporter.display_messages(None)
    report_result = json.loads(output.getvalue())
    return report_result



# pylint: disable=missing-docstring
"""Regression test for https://github.com/PyCQA/pylint/issues/1168

The problem was that we weren't handling keyword-only arguments annotations,
which means we were never processing them.
"""

import re
import typing
from collections import Counter as CollectionCounter
from collections import defaultdict
from datetime import datetime
from typing import (
    Any,
    Callable,
    Iterable,
    List,
    NamedTuple,
    Optional,
    Pattern,
    Sequence,
    Set,
    Tuple,
)


def func1(arg: Optional[Callable]=None):
    return arg


def func2(*, arg: Optional[Iterable]=None):
    return arg


SOME_VALUE = [1] # type: List[Any]
for VALUE in [[1], [2], [3]]:  # type: Tuple[Any]
    print(VALUE)


class ContextManager:
    def __enter__(self):
        return {1}

    def __exit__(self, *_args):
        pass


with ContextManager() as SOME_DICT: # type: Set[int]
    print(SOME_DICT)


def func_test_type_comment(param):
    # type: (NamedTuple) -> Tuple[NamedTuple, Pattern]
    return param, re.compile('good')


def typing_fully_qualified():
    variable = None  # type: typing.Optional[str]
    other_variable: 'typing.Optional[str]' = None
    return variable, other_variable


def function(arg1,  # type: Iterable
             arg2  # type: List
            ):
    # type: (...) -> Sequence
    """docstring"""
    print(arg1, arg2)


def magic(alpha, beta, gamma):
    # type: (str, Optional[str], Optional[datetime]) -> Any
    """going strong"""
    return alpha, beta, gamma


def unused_assignment_import():
    foo_or_bar = 42  # type: defaultdict
    return foo_or_bar


def unused_reassigned_import(counter):
    # type: (CollectionCounter) -> int
    print(counter)
    return 42



# pylint: disable=missing-docstring


def some_func():
    pass


def decorate(func):
    """Decorate *fn* to return ``self`` to enable chained method calls."""
    def wrapper(self, *args, **kw):
        func(self, *args, **kw)
        return 42
    return wrapper


class Class:

    def some_method(self):
        pass

    @decorate
    def some_other_decorated_method(self):
        pass

    def some_other_method(self):
        value = self.some_method()  # [assignment-from-no-return]
        other_value = self.some_other_decorated_method()
        return value + other_value


VALUE = some_func() # [assignment-from-no-return]


class Parent:
    """Parent class"""

    def compute(self):
        """This isn't supported by all child classes"""

        # pylint: disable=no-self-use
        raise ValueError('Not supported for this object')

    def test(self):
        """Test"""

        result = self.compute()
        return result


class Child(Parent):
    """Child class"""

    def compute(self):
        """This is supported for this child class"""

        return 42



#pylint: disable=W0401,W0611,no-absolute-import,invalid-name,import-error,bare-except,broad-except,wrong-import-order,ungrouped-imports,wrong-import-position
"""check unexistant names imported are reported"""
from __future__ import print_function

import collections.tutu  # [no-name-in-module]
from collections import toto  # [no-name-in-module]
toto.yo()

from xml.etree import ElementTree
ElementTree.nonexistant_function()  # [no-member]
ElementTree.another.nonexistant.function()  # [no-member]


import sys
print(sys.stdout, 'hello world')
print(sys.stdoout, 'bye bye world')  # [no-member]


import re
re.finditer('*', 'yo')

from rie import *
from re import findiiter, compiile  # [no-name-in-module,no-name-in-module]

import os
'SOMEVAR' in os.environ  # [pointless-statement]

try:
    from collections import something
except ImportError:
    something = None

try:
    from collections import anything # [no-name-in-module]
except ValueError:
    anything = None

try:
    import collections.missing
except ImportError:
    pass

try:
    import collections.indeed_missing # [no-name-in-module]
except ValueError:
    pass

try:
    import collections.emit # [no-name-in-module]
except Exception:
    pass

try:
    import collections.emit1
except ImportError:
    pass

try:
    if something:
        import collections.emit2 # [no-name-in-module]
except Exception:
    pass

from .no_self_use import Super
from .no_self_use import lala  # [no-name-in-module]
from .no_self_use.bla import lala1 # [no-name-in-module]



"""Test disabling of cyclic import check inside a function
"""
# pylint: disable=no-absolute-import,import-outside-toplevel
from __future__ import print_function


def func():
    """Test disabling of cyclic import check inside a function"""
    from . import w0401_cycle  # pylint: disable=cyclic-import
    if w0401_cycle:
        print(w0401_cycle)



# pylint: disable=missing-module-docstring, missing-class-docstring
# pylint: disable=too-few-public-methods, missing-function-docstring
# pylint: disable=import-error

import random

from unknown import Unknown


class ExampleClass():
    pass


def example_function():
    pass


def returns_str():
    return "abcd"


def returns_int():
    return 0


def returns_tuple():
    return 0, "abc"


# Might not be thorough if same hash seed is used in testing...
def returns_random_type():
    if random.randint(0, 1) > 0:
        return 0

    return "abc"

ExampleClass.__name__ = 1  # [non-str-assignment-to-dunder-name]
ExampleClass.__name__ = True  # [non-str-assignment-to-dunder-name]
ExampleClass.__name__ = returns_tuple() # [non-str-assignment-to-dunder-name]
ExampleClass.__name__ = returns_int() # [non-str-assignment-to-dunder-name]
ExampleClass.__name__ = "foo"
ExampleClass.__name__ = returns_str()
ExampleClass.__name__ = returns_random_type()
ExampleClass.__name__ = Unknown

example_function.__name__ = 1  # [non-str-assignment-to-dunder-name]
example_function.__name__ = True  # [non-str-assignment-to-dunder-name]
example_function.__name__ = returns_tuple() # [non-str-assignment-to-dunder-name]
example_function.__name__ = returns_int() # [non-str-assignment-to-dunder-name]
example_function.__name__ = "foo"
example_function.__name__ = returns_str()
example_function.__name__ = returns_random_type()
example_function.__name__ = Unknown



# pylint: disable=missing-docstring,too-few-public-methods
__version__ = "1.0"
ignored_SOME_CONSTANT = 42

explicit_bad_some_constant = 42  # [blacklisted-name]

snake_case_bad_SOME_CONSTANT = 42  # [invalid-name]


class my_class:
    def __init__(self, arg_x):
        self._my_secret_x = arg_x

    @property
    def my_public_x(self):
        return self._my_secret_x * 2


def blacklisted_2_snake_case():  # [blacklisted-name]
    pass



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

import pytest

from pylint.exceptions import InvalidMessageError, UnknownMessageError


def test_len_str(msgid_store, msgids):
    assert len(msgid_store) == len(msgids)
    str_result = str(msgid_store)
    assert "MessageIdStore: [" in str_result
    assert "  - W1234 (warning-symbol)" in str_result
    assert "  - W1235 (warning-symbol-two)" in str_result
    assert "  - C1234 (convention-symbol)" in str_result
    assert "  - E1234 (error-symbol)" in str_result
    assert "]" in str_result


def test_get_message_ids(msgid_store, msgids):
    """We can get message id even with capitalization problem."""
    msgid = list(msgids.keys())[0]
    msgids_result = msgid_store.get_active_msgids(msgid.lower())
    assert len(msgids_result) == 1
    assert msgid == msgids_result[0]


def test_get_message_ids_not_existing(empty_msgid_store):
    with pytest.raises(UnknownMessageError) as error:
        w9876 = "W9876"
        empty_msgid_store.get_active_msgids(w9876)
    assert w9876 in str(error.value)


def test_register_message_definitions(empty_msgid_store, message_definitions):
    number_of_msgid = len(message_definitions)
    for message_definition in message_definitions:
        empty_msgid_store.register_message_definition(message_definition)
        if message_definition.old_names:
            number_of_msgid += len(message_definition.old_names)
    assert len(empty_msgid_store) == number_of_msgid


def test_add_msgid_and_symbol(empty_msgid_store):
    empty_msgid_store.add_msgid_and_symbol("E1235", "new-sckiil")
    empty_msgid_store.add_legacy_msgid_and_symbol("C1235", "old-sckiil", "E1235")
    assert len(empty_msgid_store) == 2
    message_ids = empty_msgid_store.get_active_msgids("E1235")
    assert len(message_ids) == 1
    assert message_ids[0] == "E1235"
    message_ids = empty_msgid_store.get_active_msgids("old-sckiil")
    assert len(message_ids) == 1
    assert message_ids[0] == "E1235"
    assert empty_msgid_store.get_symbol("C1235") == "old-sckiil"
    assert empty_msgid_store.get_symbol("E1235") == "new-sckiil"
    assert empty_msgid_store.get_msgid("old-sckiil") == "C1235"
    assert empty_msgid_store.get_msgid("new-sckiil") == "E1235"
    with pytest.raises(KeyError):
        empty_msgid_store.get_symbol("C1234")
    with pytest.raises(KeyError):
        empty_msgid_store.get_msgid("not-exist")


def test_duplicate_symbol(empty_msgid_store):
    empty_msgid_store.add_msgid_and_symbol("W1234", "warning-symbol")
    with pytest.raises(InvalidMessageError) as error:
        empty_msgid_store.check_msgid_and_symbol("W1234", "other-symbol")
    assert (
        "Message id 'W1234' cannot have both 'other-symbol' and 'warning-symbol' as symbolic name."
        in str(error.value)
    )


def test_duplicate_msgid(msgid_store):
    msgid_store.add_msgid_and_symbol("W1234", "warning-symbol")
    with pytest.raises(InvalidMessageError) as error:
        msgid_store.check_msgid_and_symbol("W1235", "warning-symbol")
    assert (
        "Message symbol 'warning-symbol' cannot be used for 'W1234' and 'W1235'"
        in str(error.value)
    )



"""
Test that no StopIteration is raised inside a generator
"""
# pylint: disable=missing-docstring,invalid-name,import-error, try-except-raise, wrong-import-position,not-callable,raise-missing-from
import asyncio

class RebornStopIteration(StopIteration):
    """
    A class inheriting from StopIteration exception
    """

# This one is ok
def gen_ok():
    yield 1
    yield 2
    yield 3

# pylint should warn about this one
# because of a direct raising of StopIteration inside generator
def gen_stopiter():
    yield 1
    yield 2
    yield 3
    raise StopIteration  # [stop-iteration-return]

# pylint should warn about this one
# because of a direct raising of an exception inheriting from StopIteration inside generator
def gen_stopiterchild():
    yield 1
    yield 2
    yield 3
    raise RebornStopIteration  # [stop-iteration-return]

# pylint should warn here
# because of the possibility that next raises a StopIteration exception
def gen_next_raises_stopiter():
    g = gen_ok()
    while True:
        yield next(g)  # [stop-iteration-return]

# This one is the same as gen_next_raises_stopiter
# but is ok because the next function is inside
# a try/except block handling StopIteration
def gen_next_inside_try_except():
    g = gen_ok()
    while True:
        try:
            yield next(g)
        except StopIteration:
            return

# This one is the same as gen_next_inside_try_except
# but is not ok because the next function is inside
# a try/except block that don't handle StopIteration
def gen_next_inside_wrong_try_except():
    g = gen_ok()
    while True:
        try:
            yield next(g)  # [stop-iteration-return]
        except ValueError:
            return

# This one is the same as gen_next_inside_try_except
# but is not ok because the next function is inside
# a try/except block that handle StopIteration but reraise it
def gen_next_inside_wrong_try_except2():
    g = gen_ok()
    while True:
        try:
            yield next(g)
        except StopIteration:
            raise StopIteration  # [stop-iteration-return]

# Those two last are ok
def gen_in_for():
    for el in gen_ok():
        yield el

def gen_yield_from():
    yield from gen_ok()


def gen_dont_crash_on_no_exception():
    g = gen_ok()
    while True:
        try:
            yield next(g) # [stop-iteration-return]
        except ValueError:
            raise


def gen_dont_crash_on_uninferable():
    # https://github.com/PyCQA/pylint/issues/1779
    yield from iter()
    raise asyncio.TimeoutError()


# https://github.com/PyCQA/pylint/issues/1830
def gen_next_with_sentinel():
    yield next([], 42) # No bad return


from itertools import count

# https://github.com/PyCQA/pylint/issues/2158
def generator_using_next():
    counter = count()
    number = next(counter)
    yield number * 2


# pylint: disable=no-self-use,too-few-public-methods
class SomeClassWithNext:
    def next(self):
        return iter([1, 2, 3])
    def some_gen(self):
        for value in self.next():
            yield value


SomeClassWithNext().some_gen()


def something_invalid():
    raise Exception('cannot iterate this')


def invalid_object_passed_to_next():
    yield next(something_invalid()) # [stop-iteration-return]



# pylint: disable=missing-docstring,too-many-ancestors, broad-except
import collections.abc
from typing import TYPE_CHECKING, Any, MutableMapping

if TYPE_CHECKING:
    BaseClass = MutableMapping[str, Any]
else:
    BaseClass = collections.abc.MutableMapping


class TestBaseException(BaseClass):
    pass


class TestException(TestBaseException):
    pass


def test():
    try:
        1 / 0
    except TestException:  # [try-except-raise]
        raise
    except Exception:
        pass



# pylint: disable=missing-docstring,wrong-import-position

def stupid_function(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9): # [too-many-arguments]
    return arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9


class MyClass:
    text = "MyText"

    def mymethod1(self):
        return self.text

    def mymethod2(self):
        return self.mymethod1.__get__(self, MyClass)


MyClass().mymethod2()()


# Check a false positive does not occur
from functools import partial


def root_function(first, second, third):
    return first + second + third


def func_call():
    """Test we don't emit a FP for https://github.com/PyCQA/pylint/issues/2588"""
    partial_func = partial(root_function, 1, 2, 3)
    partial_func()
    return root_function(1, 2, 3)



# pylint: disable=missing-docstring, invalid-name
# https://github.com/PyCQA/pylint/issues/3165

any([])
all([])

any([0 for x in list(range(10))]) # [use-a-generator]
all([0 for y in list(range(10))]) # [use-a-generator]

any(0 for x in list(range(10)))
all(0 for y in list(range(10)))



"""Test that __future__ is not the first statement after the docstring."""
import collections
from __future__ import print_function # [misplaced-future]
from __future__ import with_statement

DATA = collections



"""Test that some if expressions can be simplified."""

# pylint: disable=missing-docstring, invalid-name


def test_simplifiable_1(arg):
    # Simple test that can be replaced by bool(arg)
    return True if arg else False # [simplifiable-if-expression]

def test_simplifiable_2(arg):
    # Simple test that can be replaced by not arg
    return False if arg else True # [simplifiable-if-expression]

def test_simplifiable_3(arg):
    # Simple test that can be replaced by arg == 1
    return True if arg == 1 else False # [simplifiable-if-expression]

def test_simplifiable_4(arg):
    # Simple test that can be replaced by not (arg == 1)
    return False if arg == 1 else True # [simplifiable-if-expression]

def test_not_simplifiable(arg):
    x = True if arg else True
    y = 0 if arg else 1
    t = False if arg != 1 else False
    t2 = None if arg > 3 else False
    return x, y, t, t2



# pylint: disable=R0903,W0232, useless-object-inheritance
"""check for scope problems"""

__revision__ = None

class Well(object):
    """well"""
    attr = 42
    get_attr = lambda arg=attr: arg * 24
    # +1: [undefined-variable, used-before-assignment]
    get_attr_bad = lambda arg=revattr: revattr * 42
    revattr = 24
    bad_lambda = lambda: get_attr_bad # [undefined-variable]
    bad_gen = list(attr + i for i in range(10)) # [undefined-variable]

    class Data(object):
        """base hidden class"""
    class Sub(Data):
        """whaou, is Data found???"""
        attr = Data() # [undefined-variable]
    def func(self):
        """check Sub is not defined here"""
        return Sub(), self # [undefined-variable]


class Right:
    """right"""
    class Result1:
        """result one"""
        OK = 0
    def work(self) -> Result1:
        """good type hint"""
        return self.Result1.OK


class Wrong:
    """wrong"""
    class Result2:
        """result two"""
        OK = 0
    def work(self) -> self.Result2: # [undefined-variable]
        """bad type hint"""
        return self.Result2.OK



# pylint: disable=missing-module-docstring, missing-function-docstring
import os
import sys
from unittest.mock import patch

import pytest

from pylint import run_epylint, run_pylint, run_pyreverse, run_symilar


@pytest.mark.parametrize(
    "runner", [run_pylint, run_epylint, run_pyreverse, run_symilar]
)
def test_runner(runner):
    filepath = os.path.abspath(__file__)
    testargs = ["", filepath]
    with patch.object(sys, "argv", testargs):
        with pytest.raises(SystemExit) as err:
            runner()
        assert err.value.code == 0



# pylint: disable=W0640
"""Check a nested function defined in a loop."""

def example(args):
    """The check"""
    for i in args:
        def nested():
            print(i)
        nested()
    for i in args:
        print(i)



# pylint: disable=missing-docstring, invalid-name, blacklisted-name, redefined-builtin, unused-variable
import sys

def foo():
    exit()  # [consider-using-sys-exit]

def foo_1():
    quit()  # [consider-using-sys-exit]

def foo_2():
    quit = 'abc'
    sys.exit()

quit()  # [consider-using-sys-exit]



# Copyright (c) 2015-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
# Copyright (c) 2018 Yury Gribov <tetra2005@gmail.com>
# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

import astroid

from pylint.checkers import strings
from pylint.testutils import CheckerTestCase, Message

TEST_TOKENS = (
    '"X"',
    "'X'",
    "'''X'''",
    '"""X"""',
    'r"X"',
    "R'X'",
    'u"X"',
    "F'X'",
    'f"X"',
    "F'X'",
    'fr"X"',
    'Fr"X"',
    'fR"X"',
    'FR"X"',
    'rf"X"',
    'rF"X"',
    'Rf"X"',
    'RF"X"',
)


class TestStringChecker(CheckerTestCase):
    CHECKER_CLASS = strings.StringFormatChecker

    def test_format_bytes(self):
        code = "b'test'.format(1, 2)"
        node = astroid.extract_node(code)
        with self.assertNoMessages():
            self.checker.visit_call(node)

    def test_format_types(self):
        for code in ("'%s' % 1", "'%d' % 1", "'%f' % 1"):
            with self.assertNoMessages():
                node = astroid.extract_node(code)
            self.checker.visit_binop(node)

        for code in (
            "'%s' % 1",
            "'%(key)s' % {'key' : 1}",
            "'%d' % 1",
            "'%(key)d' % {'key' : 1}",
            "'%f' % 1",
            "'%(key)f' % {'key' : 1}",
            "'%d' % 1.1",
            "'%(key)d' % {'key' : 1.1}",
            "'%s' % []",
            "'%(key)s' % {'key' : []}",
            "'%s' % None",
            "'%(key)s' % {'key' : None}",
        ):
            with self.assertNoMessages():
                node = astroid.extract_node(code)
                self.checker.visit_binop(node)

        for code, arg_type, format_type in [
            ("'%d' % '1'", "builtins.str", "d"),
            ("'%(key)d' % {'key' : '1'}", "builtins.str", "d"),
            ("'%x' % 1.1", "builtins.float", "x"),
            ("'%(key)x' % {'key' : 1.1}", "builtins.float", "x"),
            ("'%d' % []", "builtins.list", "d"),
            ("'%(key)d' % {'key' : []}", "builtins.list", "d"),
        ]:
            node = astroid.extract_node(code)
            with self.assertAddsMessages(
                Message(
                    "bad-string-format-type", node=node, args=(arg_type, format_type)
                )
            ):
                self.checker.visit_binop(node)


def test_str_eval():
    for token in TEST_TOKENS:
        assert strings.str_eval(token) == "X"



"""Checks that disabling 'ungrouped-imports' on an import prevents subsequent
imports from being considered ungrouped in respect to it."""
# pylint: disable=unused-import,wrong-import-position,wrong-import-order,using-constant-test
# pylint: disable=import-error
from os.path import basename
import logging.config  # pylint: disable=ungrouped-imports
import os.path
import logging
from os.path import join  # [ungrouped-imports]
import logging.handlers  # [ungrouped-imports]



#pylint: disable=missing-docstring, undefined-variable, invalid-name, too-few-public-methods, wrong-import-position,import-error

import collections
from unknown import Unknown

# Positive test cases
class A:
    pass

class B(A):
    pass

isinstance(A(), A)
isinstance(A(), B)

isinstance(-9999, int)
isinstance(True and False, bool)
isinstance("a 'string'", type("test"))

isinstance(3.123213, collections.OrderedDict)
isinstance(foo, (int, collections.Counter))
isinstance("a string", ((int, type(False)), (float, set), str))
isinstance(10, (int,) + (str, bool) + (dict, list, tuple))
isinstance(10, tuple(Unknown))

# Negative test cases
isinstance({a:1}, hash) # [isinstance-second-argument-not-valid-type]
isinstance(64, hex) # [isinstance-second-argument-not-valid-type]
isinstance({b: 100}, (hash, dict)) # [isinstance-second-argument-not-valid-type]
isinstance("string", ((dict, iter), str, (int, bool))) # [isinstance-second-argument-not-valid-type]
isinstance(int, 1) # [isinstance-second-argument-not-valid-type]



"""Make sure no-member is not emitted when modifying __doc__ via augmented assignment

https://github.com/PyCQA/pylint/issues/1078
"""
# pylint: disable=too-few-public-methods,missing-class-docstring
class Cls:
    def test(self):
        "a"

    test.__doc__ += "b"


print(Cls().test.__doc__)
print(Cls.test.__doc__)



# pylint: disable=missing-docstring,invalid-name,too-few-public-methods
# https://github.com/PyCQA/pylint/issues/2439
class TestClass:
    __slots__ = ["_i"]

    def __init__(self):
        self._i = 0

    @property
    def i(self):
        return self._i

    @i.setter
    def i(self, v):
        self._i = v

    other = i


instance = TestClass()
instance.other = 42
print(instance.i)



"""Checks the maximum block level is smaller than 6 in function definitions"""

#pylint: disable=using-constant-test, missing-docstring, too-many-return-statements,no-else-return

def my_function():
    if 1:  # [too-many-nested-blocks]
        for i in range(10):
            if i == 2:
                while True:
                    try:
                        if True:
                            i += 1
                    except IOError:
                        pass

    if 1:
        for i in range(10):
            if i == 2:
                while True:
                    try:
                        i += 1
                    except IOError:
                        pass

    def nested_func():
        if True:
            for i in range(10):
                while True:
                    if True:
                        if True:
                            yield i

    nested_func()

def more_complex_function():
    attr1 = attr2 = attr3 = [1, 2, 3]
    if attr1:
        for i in attr1:
            if attr2:
                return i
            else:
                return 'duh'
    elif attr2:
        for i in attr2:
            if attr2:
                return i
            else:
                return 'duh'
    else:
        for i in range(15):
            if attr3:
                return i
            else:
                return 'doh'
            return None

def elif_function():
    arg = None
    if arg == 1:
        return 1
    elif arg == 2:
        return 2
    elif arg == 3:
        return 3
    elif arg == 4:
        return 4
    elif arg == 5:
        return 5
    elif arg == 6:
        return 6
    elif arg == 7:
        return 7
    return None

def else_if_function():
    arg = None
    if arg == 1:  # [too-many-nested-blocks]
        return 1
    else:
        if arg == 2:
            return 2
        else:
            if arg == 3:
                return 3
            else:
                if arg == 4:
                    return 4
                else:
                    if arg == 5:
                        return 5
                    else:
                        if arg == 6:
                            return 6
                        else:
                            if arg == 7:
                                return 7
        return None



"""Check invalid value returned by __getnewargs__ """

# pylint: disable=too-few-public-methods,missing-docstring,no-self-use,import-error, useless-object-inheritance
import six

from missing import Missing


class FirstGoodGetNewArgs(object):
    """__getnewargs__ returns <type 'tuple'>"""

    def __getnewargs__(self):
        return (1, "2", 3)


class SecondGoodGetNewArgs(object):
    """__getnewargs__ returns <type 'tuple'>"""

    def __getnewargs__(self):
        return tuple()


class GetNewArgsMetaclass(type):
    def __getnewargs__(cls):
        return (1, 2, 3)


@six.add_metaclass(GetNewArgsMetaclass)
class ThirdGoodGetNewArgs(object):
    """GetNewArgs through the metaclass."""


class FirstBadGetNewArgs(object):
    """ __getnewargs__ returns an integer """

    def __getnewargs__(self):  # [invalid-getnewargs-returned]
        return 1


class SecondBadGetNewArgs(object):
    """ __getnewargs__ returns str """

    def __getnewargs__(self):  # [invalid-getnewargs-returned]
        return "(1, 2, 3)"


class ThirdBadGetNewArgs(object):
    """ __getnewargs__ returns node which does not have 'value' in AST """

    def __getnewargs__(self):  # [invalid-getnewargs-returned]
        return lambda: tuple(1, 2)


class AmbigousGetNewArgs(object):
    """ Uninferable return value """
    __getnewargs__ = lambda self: Missing


class AnotherAmbiguousGetNewArgs(object):
    """Potential uninferable return value"""
    def __getnewargs__(self):
        return tuple(Missing)



# pylint: disable=missing-docstring,too-few-public-methods

import asyncio


async def bla1():
    await asyncio.sleep(1)


async def bla2():
    await asyncio.sleep(2)


async def combining_coroutine1():
    await bla1()
    await bla2()


class Coro:
    async def func(self):
        future1 = bla1()
        future2 = bla2()
        await asyncio.gather(future1, future2)


async def combining_coroutine2():
    future1 = bla1()
    future2 = bla2()
    future3 = Coro().func()
    await asyncio.gather(future1, future2, future3)


def do_stuff():
    loop = asyncio.get_event_loop()
    loop.run_until_complete(combining_coroutine1())
    loop.run_until_complete(combining_coroutine2())



"""Test that boolean conditions simplify to a constant value"""
# pylint: disable=pointless-statement
from unknown import Unknown  # pylint: disable=import-error


def func(_):
    """Pointless function"""


CONSTANT = 100
OTHER = 200

# Simplifies any boolean expression that is coerced into a True/False value
bool(CONSTANT or True)  # [condition-evals-to-constant]
assert CONSTANT or True  # [condition-evals-to-constant]
if CONSTANT and False:  # [condition-evals-to-constant]
    pass
elif CONSTANT and False:  # [condition-evals-to-constant]
    pass
while CONSTANT and False:  # [condition-evals-to-constant]
    break
1 if CONSTANT or True else 2  # [condition-evals-to-constant]
z = [x for x in range(10) if x or True]  # [condition-evals-to-constant]

# Simplifies recursively
assert True or CONSTANT or OTHER  # [condition-evals-to-constant]
assert (CONSTANT or True) or (CONSTANT or True)  # [condition-evals-to-constant]

# Will try to infer the truthiness of an expression as long as it doesn't contain any variables
assert 3 + 4 or CONSTANT  # [condition-evals-to-constant]
assert Unknown or True  # [condition-evals-to-constant]

assert True or True  # [condition-evals-to-constant]
assert False or False  # [condition-evals-to-constant]
assert True and True  # [condition-evals-to-constant]
assert False and False  # [condition-evals-to-constant]


# A bare constant that's not inside of a boolean operation will emit `using-constant-test` instead
if True:  # pylint: disable=using-constant-test
    pass

# Expressions not in one of the above situations will not emit a message
CONSTANT or True
bool(CONSTANT or OTHER)
bool(func(CONSTANT or True))



# pylint: disable=missing-module-docstring, missing-function-docstring, protected-access
import unittest.mock

import pylint.lint


def check_configuration_file_reader(config_file):
    """Initialize pylint with the given configuration file and check that
    what we initialized the linter with what was expected.
    """
    args = ["--rcfile", str(config_file), __file__]
    # If we used `pytest.raises(SystemExit)`, the `runner` variable
    # would not be accessible outside the `with` block.
    with unittest.mock.patch("sys.exit") as mocked_exit:
        # Do not actually run checks, that could be slow. Do not mock
        # `Pylinter.check`: it calls `Pylinter.initialize` which is
        # needed to properly set up messages inclusion/exclusion
        # in `_msg_states`, used by `is_message_enabled`.
        with unittest.mock.patch("pylint.lint.pylinter.check_parallel"):
            runner = pylint.lint.Run(args)

    # "logging-not-lazy" and "logging-format-interpolation"
    expected_disabled = {"W1201", "W1202"}
    for msgid in expected_disabled:
        assert not runner.linter.is_message_enabled(msgid)
    assert runner.linter.config.jobs == 10
    assert runner.linter.config.reports

    mocked_exit.assert_called_once_with(0)
    return runner


def test_can_read_ini(tmp_path):
    # Check that we can read the "regular" INI .pylintrc file
    config_file = tmp_path / ".pylintrc"
    config_file.write_text(
        """
[messages control]
disable = logging-not-lazy,logging-format-interpolation
jobs = 10
reports = yes
"""
    )
    check_configuration_file_reader(config_file)


def test_can_read_setup_cfg(tmp_path):
    # Check that we can read a setup.cfg (which is an INI file where
    # section names are prefixed with "pylint."
    config_file = tmp_path / "setup.cfg"
    config_file.write_text(
        """
[pylint.messages control]
disable = logging-not-lazy,logging-format-interpolation
jobs = 10
reports = yes
"""
    )
    check_configuration_file_reader(config_file)


def test_can_read_toml(tmp_path):
    # Check that we can read a TOML file where lists and integers are
    # expressed as strings.
    config_file = tmp_path / "pyproject.toml"
    config_file.write_text(
        """
[tool.pylint."messages control"]
disable = "logging-not-lazy,logging-format-interpolation"
jobs = "10"
reports = "yes"
"""
    )
    check_configuration_file_reader(config_file)


def test_can_read_toml_rich_types(tmp_path):
    # Check that we can read a TOML file where lists, integers and
    # booleans are expressed as such (and not as strings), using TOML
    # type system.
    config_file = tmp_path / "pyproject.toml"
    config_file.write_text(
        """
[tool.pylint."messages control"]
disable = [
    "logging-not-lazy",
    "logging-format-interpolation",
]
jobs = 10
reports = true
"""
    )
    check_configuration_file_reader(config_file)



"""Check invalid value returned by __repr__ """

# pylint: disable=too-few-public-methods,missing-docstring,no-self-use,import-error, useless-object-inheritance
import six

from missing import Missing


class FirstGoodRepr(object):
    """__repr__ returns <type 'str'>"""

    def __repr__(self):
        return "some repr"


class SecondGoodRepr(object):
    """__repr__ returns <type 'str'>"""

    def __repr__(self):
        return str(123)


class ReprMetaclass(type):
    def __repr__(cls):
        return "some repr"


@six.add_metaclass(ReprMetaclass)
class ThirdGoodRepr(object):
    """Repr through the metaclass."""


class FirstBadRepr(object):
    """ __repr__ returns bytes """

    def __repr__(self):  # [invalid-repr-returned]
        return b"123"


class SecondBadRepr(object):
    """ __repr__ returns int """

    def __repr__(self):  # [invalid-repr-returned]
        return 1


class ThirdBadRepr(object):
    """ __repr__ returns node which does not have 'value' in AST """

    def __repr__(self):  # [invalid-repr-returned]
        return lambda: "some repr"


class AmbiguousRepr(object):
    """ Uninferable return value """

    __repr__ = lambda self: Missing


class AnotherAmbiguousRepr(object):
    """Potential uninferable return value"""

    def __repr__(self):
        return str(Missing)



# pylint: disable=missing-docstring,no-init,unused-argument,invalid-name,too-few-public-methods

class A:
    def __init__(self):
        self.store = {}

    def get(self, key, default=None):
        return self.store.get(key, default)

class B(A):
    def get_memo(self, obj):
        return super().get(obj)



"""Check trailing comma one element tuples."""
# pylint: disable=missing-docstring
AAA = 1, # [trailing-comma-tuple]
BBB = "aaaa", # [trailing-comma-tuple]
CCC="aaa", # [trailing-comma-tuple]
FFF=['f'], # [trailing-comma-tuple]

BBB = 1, 2
CCC = (1, 2, 3)
DDD = (
    1, 2, 3,
)
EEE = (
    "aaa",
)


def test(*args, **kwargs):
    return args, kwargs


test(widget=1, label='test')
test(widget=1,
     label='test')
test(widget=1, \
     label='test')


def some_func(first, second):
    if first:
        return first, # [trailing-comma-tuple]
    if second:
        return (first, second,)
    return first, second,  # [trailing-comma-tuple]


def some_other_func():
    yield 'hello',  # [trailing-comma-tuple]

GGG = ["aaa"
], # [trailing-comma-tuple]

HHH = ["aaa"
]

III = some_func(0,
    0), # [trailing-comma-tuple]

JJJ = some_func(0,
    0)



# Copyright (c) 2016-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2016-2017 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
# Copyright (c) 2020 Damien Baty <damien.baty@polyconseil.fr>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Tests for the pylint checker in :mod:`pylint.extensions.bad_builtin
"""
from os import path as osp

import pytest

from pylint.extensions.bad_builtin import BadBuiltinChecker
from pylint.lint import fix_import_path

EXPECTED = [
    "Used builtin function 'map'. Using a list comprehension can be clearer.",
    "Used builtin function 'filter'. Using a list comprehension can be clearer.",
]


@pytest.fixture(scope="module")
def checker():
    return BadBuiltinChecker


@pytest.fixture(scope="module")
def disable():
    return ["I"]


def test_types_redefined(linter):
    elif_test = osp.join(osp.dirname(osp.abspath(__file__)), "data", "bad_builtin.py")
    with fix_import_path([elif_test]):
        linter.check([elif_test])
    msgs = sorted(linter.reporter.messages, key=lambda item: item.line)
    assert len(msgs) == 2
    for msg, expected in zip(msgs, EXPECTED):
        assert msg.symbol == "bad-builtin"
        assert msg.msg == expected



# pylint: disable=too-few-public-methods,import-error, no-absolute-import,missing-docstring, misplaced-comparison-constant
# pylint: disable=useless-super-delegation,wrong-import-position,invalid-name, wrong-import-order, condition-evals-to-constant

if len('TEST'):  # [len-as-condition]
    pass

if not len('TEST'):  # [len-as-condition]
    pass

z = []
if z and len(['T', 'E', 'S', 'T']):  # [len-as-condition]
    pass

if True or len('TEST'):  # [len-as-condition]
    pass

if len('TEST') == 0:  # Should be fine
    pass

if len('TEST') < 1:  # Should be fine
    pass

if len('TEST') <= 0:  # Should be fine
    pass

if 1 > len('TEST'):  # Should be fine
    pass

if 0 >= len('TEST'):  # Should be fine
    pass

if z and len('TEST') == 0:  # Should be fine
    pass

if 0 == len('TEST') < 10:  # Should be fine
    pass

if 0 < 1 <= len('TEST') < 10:  # Should be fine
    pass

if 10 > len('TEST') != 0:  # Should be fine
    pass

if 10 > len('TEST') > 1 > 0:  # Should be fine
    pass

if 0 <= len('TEST') < 100:  # Should be fine
    pass

if z or 10 > len('TEST') != 0:  # Should be fine
    pass

if z:
    pass
elif len('TEST'):  # [len-as-condition]
    pass

if z:
    pass
elif not len('TEST'):  # [len-as-condition]
    pass

while len('TEST'):  # [len-as-condition]
    pass

while not len('TEST'):  # [len-as-condition]
    pass

while z and len('TEST'):  # [len-as-condition]
    pass

while not len('TEST') and z:  # [len-as-condition]
    pass

assert len('TEST') > 0  # Should be fine

x = 1 if len('TEST') != 0 else 2  # Should be fine

f_o_o = len('TEST') or 42  # Should be fine

a = x and len(x)  # Should be fine

def some_func():
    return len('TEST') > 0  # Should be fine

def github_issue_1325():
    l = [1, 2, 3]
    length = len(l) if l else 0  # Should be fine
    return length

def github_issue_1331(*args):
    assert False, len(args)  # Should be fine

def github_issue_1331_v2(*args):
    assert len(args), args  # [len-as-condition]

def github_issue_1331_v3(*args):
    assert len(args) or z, args  # [len-as-condition]

def github_issue_1331_v4(*args):
    assert z and len(args), args  # [len-as-condition]

b = bool(len(z)) # [len-as-condition]
c = bool(len('TEST') or 42) # [len-as-condition]


def github_issue_1879():

    class ClassWithBool(list):
        def __bool__(self):
            return True

    class ClassWithoutBool(list):
        pass

    class ChildClassWithBool(ClassWithBool):
        pass

    class ChildClassWithoutBool(ClassWithoutBool):
        pass

    assert len(ClassWithBool())
    assert len(ChildClassWithBool())
    assert len(ClassWithoutBool())  # [len-as-condition]
    assert len(ChildClassWithoutBool())  # [len-as-condition]
    assert len(range(0))  # [len-as-condition]
    assert len([t + 1 for t in []])  # [len-as-condition]
    assert len(u + 1 for u in [])  # [len-as-condition]
    assert len({"1":(v + 1) for v in {}})  # [len-as-condition]
    assert len(set((w + 1) for w in set()))  # [len-as-condition]

    # pylint: disable=import-outside-toplevel
    import numpy
    numpy_array = numpy.array([0])
    if len(numpy_array) > 0:
        print('numpy_array')
    if len(numpy_array):
        print('numpy_array')
    if numpy_array:
        print('b')

    import pandas as pd
    pandas_df = pd.DataFrame()
    if len(pandas_df):
        print("this works, but pylint tells me not to use len() without comparison")
    if len(pandas_df) > 0:
        print("this works and pylint likes it, but it's not the solution intended by PEP-8")
    if pandas_df:
        print("this does not work (truth value of dataframe is ambiguous)")

    def function_returning_list(r):
        if r==1:
            return [1]
        return [2]

    def function_returning_int(r):
        if r==1:
            return 1
        return 2

    # def function_returning_generator(r):
    #     for i in [r, 1, 2, 3]:
    #         yield i

    # def function_returning_comprehension(r):
    #     return [x+1 for x in [r, 1, 2, 3]]

    # def function_returning_function(r):
    #     return function_returning_generator(r)

    assert len(function_returning_list(z))  # [len-as-condition]
    assert len(function_returning_int(z))
    # This should raise a len-as-conditions once astroid can infer it
    # See https://github.com/PyCQA/pylint/pull/3821#issuecomment-743771514
    # assert len(function_returning_generator(z))
    # assert len(function_returning_comprehension(z))
    # assert len(function_returning_function(z))



# Copyright (c) 2013-2014, 2016-2020 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2013-2014 Google, Inc.
# Copyright (c) 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2016 glegoux <gilles.legoux@gmail.com>
# Copyright (c) 2018 Rogalski, Lukasz <lukasz.rogalski@intel.com>
# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Tests for the misc checker."""

from pylint.checkers import misc
from pylint.testutils import CheckerTestCase, Message, _tokenize_str, set_config


class TestFixme(CheckerTestCase):
    CHECKER_CLASS = misc.EncodingChecker

    def test_fixme_with_message(self):
        code = """a = 1
                # FIXME message
                """
        with self.assertAddsMessages(
            Message(msg_id="fixme", line=2, args="FIXME message")
        ):
            self.checker.process_tokens(_tokenize_str(code))

    def test_todo_without_message(self):
        code = """a = 1
                # TODO
                """
        with self.assertAddsMessages(Message(msg_id="fixme", line=2, args="TODO")):
            self.checker.process_tokens(_tokenize_str(code))

    def test_xxx_without_space(self):
        code = """a = 1
                #XXX
                """
        with self.assertAddsMessages(Message(msg_id="fixme", line=2, args="XXX")):
            self.checker.process_tokens(_tokenize_str(code))

    def test_xxx_middle(self):
        code = """a = 1
                # midle XXX
                """
        with self.assertNoMessages():
            self.checker.process_tokens(_tokenize_str(code))

    def test_without_space_fixme(self):
        code = """a = 1
                #FIXME
                """
        with self.assertAddsMessages(Message(msg_id="fixme", line=2, args="FIXME")):
            self.checker.process_tokens(_tokenize_str(code))

    @set_config(notes=[])
    def test_absent_codetag(self):
        code = """a = 1
                # FIXME	                # FIXME
                # TODO	                # TODO
                # XXX	                # XXX
                """
        with self.assertNoMessages():
            self.checker.process_tokens(_tokenize_str(code))

    @set_config(notes=["CODETAG"])
    def test_other_present_codetag(self):
        code = """a = 1
                # CODETAG
                # FIXME
                """
        with self.assertAddsMessages(Message(msg_id="fixme", line=2, args="CODETAG")):
            self.checker.process_tokens(_tokenize_str(code))

    def test_issue_2321_should_not_trigger(self):
        code = 'print("# TODO this should not trigger a fixme")'
        with self.assertNoMessages():
            self.checker.process_tokens(_tokenize_str(code))

    def test_issue_2321_should_trigger(self):
        code = "# TODO this should not trigger a fixme"
        with self.assertAddsMessages(
            Message(msg_id="fixme", line=1, args="TODO this should not trigger a fixme")
        ):
            self.checker.process_tokens(_tokenize_str(code))

    def test_dont_trigger_on_todoist(self):
        code = """
        # Todoist API: What is this task about?
        # Todoist API: Look up a task's due date
        # Todoist API: Look up a Project/Label/Task ID
        # Todoist API: Fetch all labels
        # Todoist API: "Name" value
        # Todoist API: Get a task's priority
        # Todoist API: Look up the Project ID a Task belongs to
        # Todoist API: Fetch all Projects
        # Todoist API: Fetch all Tasks
        """
        with self.assertNoMessages():
            self.checker.process_tokens(_tokenize_str(code))



"""Check invalid value returned by __bytes__ """

# pylint: disable=too-few-public-methods,missing-docstring,no-self-use,import-error, useless-object-inheritance
import six

from missing import Missing


class FirstGoodBytes(object):
    """__bytes__ returns <type 'bytes'>"""

    def __bytes__(self):
        return b"some bytes"


class SecondGoodBytes(object):
    """__bytes__ returns <type 'bytes'>"""

    def __bytes__(self):
        return bytes("123", "ascii")


class BytesMetaclass(type):
    def __bytes__(cls):
        return b"some bytes"


@six.add_metaclass(BytesMetaclass)
class ThirdGoodBytes(object):
    """Bytes through the metaclass."""


class FirstBadBytes(object):
    """ __bytes__ returns bytes """

    def __bytes__(self):  # [invalid-bytes-returned]
        return "123"


class SecondBadBytes(object):
    """ __bytes__ returns int """

    def __bytes__(self):  # [invalid-bytes-returned]
        return 1


class ThirdBadBytes(object):
    """ __bytes__ returns node which does not have 'value' in AST """

    def __bytes__(self):  # [invalid-bytes-returned]
        return lambda: b"some bytes"


class AmbiguousBytes(object):
    """ Uninferable return value """

    __bytes__ = lambda self: Missing


class AnotherAmbiguousBytes(object):
    """Potential uninferable return value"""

    def __bytes__(self):
        return bytes(Missing)



"""Test typing.TypedDict"""
# pylint: disable=invalid-name,missing-class-docstring,too-few-public-methods
import typing
from typing import TypedDict


class CustomTD(TypedDict):
    var: int

class CustomTD2(typing.TypedDict, total=False):
    var2: str

class CustomTD3(CustomTD2):
    var3: int

CustomTD4 = TypedDict("CustomTD4", var4=bool)

CustomTD5 = TypedDict("CustomTD5", {"var5": bool})



""" Profiles basic -jX functionality """
# Copyright (c) 2020-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
# Copyright (c) 2020 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2020 Frank Harrison <frank@doublethefish.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

# pylint: disable=protected-access,missing-function-docstring,no-self-use

import os
import pprint
import time

import pytest

import pylint.interfaces
from pylint.checkers.base_checker import BaseChecker
from pylint.lint import PyLinter, Run, check_parallel
from pylint.testutils import GenericTestReporter as Reporter
from pylint.utils import register_plugins


def _empty_filepath():
    return os.path.abspath(
        os.path.join(
            os.path.dirname(__file__), "..", "input", "benchmark_minimal_file.py"
        )
    )


class SleepingChecker(BaseChecker):
    """A checker that sleeps, the wall-clock time should reduce as we add workers

    As we apply a roughly constant amount of "work" in this checker any variance is
    likely to be caused by the pylint system."""

    __implements__ = (pylint.interfaces.IRawChecker,)

    name = "sleeper"
    msgs = {
        "R9999": (
            "Test",
            "test-check",
            "Some helpful text.",
        )
    }
    sleep_duration = 0.5  # the time to pretend we're doing work for

    def process_module(self, _astroid):
        """Sleeps for `sleep_duration` on each call

        This effectively means each file costs ~`sleep_duration`+framework overhead"""
        time.sleep(self.sleep_duration)


class SleepingCheckerLong(BaseChecker):
    """A checker that sleeps, the wall-clock time should reduce as we add workers

    As we apply a roughly constant amount of "work" in this checker any variance is
    likely to be caused by the pylint system."""

    __implements__ = (pylint.interfaces.IRawChecker,)

    name = "long-sleeper"
    msgs = {
        "R9999": (
            "Test",
            "test-check",
            "Some helpful text.",
        )
    }
    sleep_duration = 0.5  # the time to pretend we're doing work for

    def process_module(self, _astroid):
        """Sleeps for `sleep_duration` on each call

        This effectively means each file costs ~`sleep_duration`+framework overhead"""
        time.sleep(self.sleep_duration)


class NoWorkChecker(BaseChecker):
    """ A checker that sleeps, the wall-clock time should change as we add threads """

    __implements__ = (pylint.interfaces.IRawChecker,)

    name = "sleeper"
    msgs = {
        "R9999": (
            "Test",
            "test-check",
            "Some helpful text.",
        )
    }

    def process_module(self, _astroid):
        pass


@pytest.mark.benchmark(
    group="baseline",
)
class TestEstablishBaselineBenchmarks:
    """Naive benchmarks for the high-level pylint framework

    Because this benchmarks the fundemental and common parts and changes seen here will
    impact everything else"""

    empty_filepath = _empty_filepath()
    empty_file_info = (
        "name-emptyfile-file",
        _empty_filepath(),
        "modname-emptyfile-mod",
    )
    lot_of_files = 500

    def test_baseline_benchmark_j1(self, benchmark):
        """Establish a baseline of pylint performance with no work

        We will add extra Checkers in other benchmarks.

        Because this is so simple, if this regresses something very serious has happened
        """
        linter = PyLinter(reporter=Reporter())
        fileinfos = [self.empty_filepath]  # Single file to end-to-end the system
        assert linter.config.jobs == 1
        assert len(linter._checkers) == 1, "Should just have 'master'"
        benchmark(linter.check, fileinfos)
        assert (
            linter.msg_status == 0
        ), "Expected no errors to be thrown: %s" % pprint.pformat(
            linter.reporter.messages
        )

    def test_baseline_benchmark_j10(self, benchmark):
        """Establish a baseline of pylint performance with no work across threads

        Same as `test_baseline_benchmark_j1` but we use -j10 with 10 fake files to
        ensure end-to-end-system invoked.

        Because this is also so simple, if this regresses something very serious has
        happened.
        """
        linter = PyLinter(reporter=Reporter())
        linter.config.jobs = 10

        # Create file per worker, using all workers
        fileinfos = [self.empty_filepath for _ in range(linter.config.jobs)]

        assert linter.config.jobs == 10
        assert len(linter._checkers) == 1, "Should have 'master'"
        benchmark(linter.check, fileinfos)
        assert (
            linter.msg_status == 0
        ), "Expected no errors to be thrown: %s" % pprint.pformat(
            linter.reporter.messages
        )

    def test_baseline_benchmark_check_parallel_j10(self, benchmark):
        """ Should demonstrate times very close to `test_baseline_benchmark_j10` """
        linter = PyLinter(reporter=Reporter())

        # Create file per worker, using all workers
        fileinfos = [self.empty_file_info for _ in range(linter.config.jobs)]

        assert len(linter._checkers) == 1, "Should have 'master'"
        benchmark(check_parallel, linter, jobs=10, files=fileinfos)
        assert (
            linter.msg_status == 0
        ), "Expected no errors to be thrown: %s" % pprint.pformat(
            linter.reporter.messages
        )

    def test_baseline_lots_of_files_j1(self, benchmark):
        """Establish a baseline with only 'master' checker being run in -j1

        We do not register any checkers except the default 'master', so the cost is just
        that of the system with a lot of files registerd"""
        if benchmark.disabled:
            benchmark(print, "skipping, only benchmark large file counts")
            return  # _only_ run this test is profiling
        linter = PyLinter(reporter=Reporter())
        linter.config.jobs = 1
        fileinfos = [self.empty_filepath for _ in range(self.lot_of_files)]
        assert linter.config.jobs == 1
        assert len(linter._checkers) == 1, "Should have 'master'"
        benchmark(linter.check, fileinfos)
        assert (
            linter.msg_status == 0
        ), "Expected no errors to be thrown: %s" % pprint.pformat(
            linter.reporter.messages
        )

    def test_baseline_lots_of_files_j10(self, benchmark):
        """Establish a baseline with only 'master' checker being run in -j10

        As with the -j1 variant above `test_baseline_lots_of_files_j1`, we do not
        register any checkers except the default 'master', so the cost is just that of
        the check_parallel system across 10 workers, plus the overhead of PyLinter"""
        if benchmark.disabled:
            benchmark(print, "skipping, only benchmark large file counts")
            return  # _only_ run this test is profiling
        linter = PyLinter(reporter=Reporter())
        linter.config.jobs = 10
        fileinfos = [self.empty_filepath for _ in range(self.lot_of_files)]
        assert linter.config.jobs == 10
        assert len(linter._checkers) == 1, "Should have 'master'"
        benchmark(linter.check, fileinfos)
        assert (
            linter.msg_status == 0
        ), "Expected no errors to be thrown: %s" % pprint.pformat(
            linter.reporter.messages
        )

    def test_baseline_lots_of_files_j1_empty_checker(self, benchmark):
        """Baselines pylint for a single extra checker being run in -j1, for N-files

        We use a checker that does no work, so the cost is just that of the system at
        scale"""
        if benchmark.disabled:
            benchmark(print, "skipping, only benchmark large file counts")
            return  # _only_ run this test is profiling
        linter = PyLinter(reporter=Reporter())
        linter.config.jobs = 1
        linter.register_checker(NoWorkChecker(linter))
        fileinfos = [self.empty_filepath for _ in range(self.lot_of_files)]
        assert linter.config.jobs == 1
        assert len(linter._checkers) == 2, "Should have 'master' and 'sleeper'"
        benchmark(linter.check, fileinfos)
        assert (
            linter.msg_status == 0
        ), "Expected no errors to be thrown: %s" % pprint.pformat(
            linter.reporter.messages
        )

    def test_baseline_lots_of_files_j10_empty_checker(self, benchmark):
        """Baselines pylint for a single extra checker being run in -j10, for N-files

        We use a checker that does no work, so the cost is just that of the system at
        scale, across workers"""
        if benchmark.disabled:
            benchmark(print, "skipping, only benchmark large file counts")
            return  # _only_ run this test is profiling
        linter = PyLinter(reporter=Reporter())
        linter.config.jobs = 10
        linter.register_checker(NoWorkChecker(linter))
        fileinfos = [self.empty_filepath for _ in range(self.lot_of_files)]
        assert linter.config.jobs == 10
        assert len(linter._checkers) == 2, "Should have 'master' and 'sleeper'"
        benchmark(linter.check, fileinfos)
        assert (
            linter.msg_status == 0
        ), "Expected no errors to be thrown: %s" % pprint.pformat(
            linter.reporter.messages
        )

    def test_baseline_benchmark_j1_single_working_checker(self, benchmark):
        """Establish a baseline of single-worker performance for PyLinter

        Here we mimick a single Checker that does some work so that we can see the
        impact of running a simple system with -j1 against the same system with -j10.

        We expect this benchmark to take very close to
        `numfiles*SleepingChecker.sleep_duration`"""
        if benchmark.disabled:
            benchmark(print, "skipping, do not want to sleep in main tests")
            return  # _only_ run this test is profiling
        linter = PyLinter(reporter=Reporter())
        linter.register_checker(SleepingChecker(linter))

        # Check the same number of files as
        # `test_baseline_benchmark_j10_single_working_checker`
        fileinfos = [self.empty_filepath for _ in range(10)]

        assert linter.config.jobs == 1
        assert len(linter._checkers) == 2, "Should have 'master' and 'sleeper'"
        benchmark(linter.check, fileinfos)
        assert (
            linter.msg_status == 0
        ), "Expected no errors to be thrown: %s" % pprint.pformat(
            linter.reporter.messages
        )

    def test_baseline_benchmark_j10_single_working_checker(self, benchmark):
        """Establishes baseline of multi-worker performance for PyLinter/check_parallel

        We expect this benchmark to take less time that test_baseline_benchmark_j1,
        `error_margin*(1/J)*(numfiles*SleepingChecker.sleep_duration)`

        Because of the cost of the framework and system the performance difference will
        *not* be 1/10 of -j1 versions."""
        if benchmark.disabled:
            benchmark(print, "skipping, do not want to sleep in main tests")
            return  # _only_ run this test is profiling
        linter = PyLinter(reporter=Reporter())
        linter.config.jobs = 10
        linter.register_checker(SleepingChecker(linter))

        # Check the same number of files as
        # `test_baseline_benchmark_j1_single_working_checker`
        fileinfos = [self.empty_filepath for _ in range(10)]

        assert linter.config.jobs == 10
        assert len(linter._checkers) == 2, "Should have 'master' and 'sleeper'"
        benchmark(linter.check, fileinfos)
        assert (
            linter.msg_status == 0
        ), "Expected no errors to be thrown: %s" % pprint.pformat(
            linter.reporter.messages
        )

    def test_baseline_benchmark_j1_all_checks_single_file(self, benchmark):
        """Runs a single file, with -j1, against all plug-ins

        ... that's the intent at least.
        """
        # Just 1 file, but all Checkers/Extensions
        fileinfos = [self.empty_filepath]

        runner = benchmark(Run, fileinfos, reporter=Reporter(), exit=False)
        assert runner.linter.config.jobs == 1
        print("len(runner.linter._checkers)", len(runner.linter._checkers))
        assert len(runner.linter._checkers) > 1, "Should have more than 'master'"

        assert (
            runner.linter.msg_status == 0
        ), "Expected no errors to be thrown: %s" % pprint.pformat(
            runner.linter.reporter.messages
        )

    def test_baseline_benchmark_j1_all_checks_lots_of_files(self, benchmark):
        """Runs lots of files, with -j1, against all plug-ins

        ... that's the intent at least.
        """
        if benchmark.disabled:
            benchmark(print, "skipping, only benchmark large file counts")
            return  # _only_ run this test is profiling
        linter = PyLinter()

        # Register all checkers/extensions and enable them
        register_plugins(
            linter, os.path.abspath(os.path.join(os.path.dirname(__file__), "..", ".."))
        )
        linter.load_default_plugins()
        linter.enable("all")

        # Just 1 file, but all Checkers/Extensions
        fileinfos = [self.empty_filepath for _ in range(self.lot_of_files)]

        assert linter.config.jobs == 1
        print("len(linter._checkers)", len(linter._checkers))
        assert len(linter._checkers) > 1, "Should have more than 'master'"
        benchmark(linter.check, fileinfos)



# Copyright (c) 2014-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2014 Google, Inc.
# Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
# Copyright (c) 2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
# Copyright (c) 2016 Łukasz Rogalski <rogalski.91@gmail.com>
# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
# Copyright (c) 2019 Mr. Senko <atodorov@mrsenko.com>
# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
# Copyright (c) 2020 Damien Baty <damien.baty@polyconseil.fr>
# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
# Copyright (c) 2020 bernie gray <bfgray3@users.noreply.github.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Functional full-module tests for PyLint."""
import csv
import os
import sys

import pytest

from pylint import testutils
from pylint.testutils import UPDATE_FILE, UPDATE_OPTION
from pylint.utils import HAS_ISORT_5

# Notes:
# - for the purpose of this test, the confidence levels HIGH and UNDEFINED
#   are treated as the same.

# TODOs
#  - implement exhaustivity tests


class LintModuleOutputUpdate(testutils.LintModuleTest):
    """If message files should be updated instead of checked."""

    class TestDialect(csv.excel):
        delimiter = ":"
        lineterminator = "\n"

    csv.register_dialect("test", TestDialect)

    def _get_expected(self):
        with self._open_source_file() as f:
            expected_msgs = self.get_expected_messages(f)
        return expected_msgs, []

    def _check_output_text(self, _, expected_output, actual_output):
        if expected_output == actual_output:
            return
        with open(self._test_file.expected_output, "w") as f:
            writer = csv.writer(f, dialect="test")
            for line in actual_output:
                writer.writerow(line.to_csv())


def get_tests():
    input_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "functional")
    suite = []
    for dirpath, _, filenames in os.walk(input_dir):
        if dirpath.endswith("__pycache__"):
            continue
        for filename in filenames:
            if filename != "__init__.py" and filename.endswith(".py"):
                # isort 5 has slightly different rules as isort 4. Testing
                # both would be hard: test with isort 5 only.
                if filename == "wrong_import_order.py" and not HAS_ISORT_5:
                    continue
                suite.append(testutils.FunctionalTestFile(dirpath, filename))
    return suite


TESTS = get_tests()
TESTS_NAMES = [t.base for t in TESTS]
TEST_WITH_EXPECTED_DEPRECATION = [
    "future_unicode_literals",
    "anomalous_unicode_escape_py3",
]


@pytest.mark.parametrize("test_file", TESTS, ids=TESTS_NAMES)
def test_functional(test_file, recwarn):
    if UPDATE_FILE.exists():
        lint_test = LintModuleOutputUpdate(test_file)
    else:
        lint_test = testutils.LintModuleTest(test_file)
    lint_test.setUp()
    lint_test._runTest()
    warning = None
    try:
        # Catch <unknown>:x: DeprecationWarning: invalid escape sequence
        # so it's not shown during tests
        warning = recwarn.pop()
    except AssertionError:
        pass
    if warning is not None:
        if (
            test_file.base in TEST_WITH_EXPECTED_DEPRECATION
            and sys.version_info.minor > 5
        ):
            assert issubclass(warning.category, DeprecationWarning)
            assert "invalid escape sequence" in str(warning.message)


if __name__ == "__main__":
    if UPDATE_OPTION in sys.argv:
        UPDATE_FILE.touch()
        sys.argv.remove(UPDATE_OPTION)
    try:
        pytest.main(sys.argv)
    finally:
        if UPDATE_FILE.exists():
            UPDATE_FILE.unlink()



"""Check invalid value returned by __getnewargs_ex__ """

# pylint: disable=too-few-public-methods,missing-docstring,no-self-use,import-error, useless-object-inheritance
import six

from missing import Missing


class FirstGoodGetNewArgsEx(object):
    """__getnewargs_ex__ returns <type 'tuple'>"""

    def __getnewargs_ex__(self):
        return ((1,), {"2": "2"})


class SecondGoodGetNewArgsEx(object):
    """__getnewargs_ex__ returns <type 'tuple'>"""

    def __getnewargs_ex__(self):
        return (tuple(), dict())


class GetNewArgsExMetaclass(type):
    def __getnewargs_ex__(cls):
        return ((1,), {"2": "2"})


@six.add_metaclass(GetNewArgsExMetaclass)
class ThirdGoodGetNewArgsEx(object):
    """GetNewArgsEx through the metaclass."""


class FirstBadGetNewArgsEx(object):
    """ __getnewargs_ex__ returns an integer """

    def __getnewargs_ex__(self):  # [invalid-getnewargs-ex-returned]
        return 1


class SecondBadGetNewArgsEx(object):
    """ __getnewargs_ex__ returns tuple with incorrect arg length"""

    def __getnewargs_ex__(self):  # [invalid-getnewargs-ex-returned]
        return (tuple(1), dict(x="y"), 1)


class ThirdBadGetNewArgsEx(object):
    """ __getnewargs_ex__ returns tuple with wrong type for first arg """

    def __getnewargs_ex__(self):  # [invalid-getnewargs-ex-returned]
        return (dict(x="y"), dict(x="y"))


class FourthBadGetNewArgsEx(object):
    """ __getnewargs_ex__ returns tuple with wrong type for second arg """

    def __getnewargs_ex__(self):  # [invalid-getnewargs-ex-returned]
        return ((1, ), (1, ))


class FifthBadGetNewArgsEx(object):
    """ __getnewargs_ex__ returns tuple with wrong type for both args """

    def __getnewargs_ex__(self):  # [invalid-getnewargs-ex-returned]
        return ({'x': 'y'}, (2,))


class SixthBadGetNewArgsEx(object):
    """ __getnewargs_ex__ returns node which does not have 'value' in AST """

    def __getnewargs_ex__(self):  # [invalid-getnewargs-ex-returned]
        return lambda: (1, 2)


class AmbigousGetNewArgsEx(object):
    """ Uninferable return value """

    __getnewargs_ex__ = lambda self: Missing


class AnotherAmbiguousGetNewArgsEx(object):
    """Potential uninferable return value"""

    def __getnewargs_ex__(self):
        return tuple(Missing)



"""Checks for if statements containing too many boolean expressions"""

# pylint: disable=invalid-name, comparison-with-itself, chained-comparison, condition-evals-to-constant

x = y = z = 5
if x > -5 and x < 5 and y > -5 and y < 5 and z > -5 and z < 5:  # [too-many-boolean-expressions]
    pass
elif True and False and 1 and 2 and 3:
    pass
elif True and False and 1 and 2 and 3 and 4 and 5: # [too-many-boolean-expressions]
    pass
elif True and (True and True) and (x == 5 or True or True): # [too-many-boolean-expressions]
    pass
elif True and (True or (x > -5 and x < 5 and (z > -5 or z < 5))): # [too-many-boolean-expressions]
    pass
elif True == True == True == True == True == True:
    pass

if True and False and 1 and 2 and 3:
    pass



""" Checks assigning attributes not found in class slots
will trigger assigning-non-slot warning.
"""
# pylint: disable=too-few-public-methods, no-init, missing-docstring, no-absolute-import, import-error, useless-object-inheritance
from collections import deque

from missing import Unknown

class Empty(object):
    """ empty """

class Bad(object):
    """ missing not in slots. """

    __slots__ = ['member']

    def __init__(self):
        self.missing = 42 # [assigning-non-slot]

class Bad2(object):
    """ missing not in slots """
    __slots__ = [deque.__name__, 'member']

    def __init__(self):
        self.deque = 42
        self.missing = 42 # [assigning-non-slot]

class Bad3(Bad):
    """ missing not found in slots """

    __slots__ = ['component']

    def __init__(self):
        self.component = 42
        self.member = 24
        self.missing = 42 # [assigning-non-slot]
        super().__init__()

class Good(Empty):
    """ missing not in slots, but Empty doesn't
    specify __slots__.
    """
    __slots__ = ['a']

    def __init__(self):
        self.missing = 42

class Good2(object):
    """ Using __dict__ in slots will be safe. """

    __slots__ = ['__dict__', 'comp']

    def __init__(self):
        self.comp = 4
        self.missing = 5

class PropertyGood(object):
    """ Using properties is safe. """

    __slots__ = ['tmp', '_value']

    @property
    def test(self):
        return self._value

    @test.setter
    def test(self, value):
        # pylint: disable=attribute-defined-outside-init
        self._value = value

    def __init__(self):
        self.test = 42

class PropertyGood2(object):
    """ Using properties in the body of the class is safe. """
    __slots__ = ['_value']

    def _getter(self):
        return self._value

    def _setter(self, value):
        # pylint: disable=attribute-defined-outside-init
        self._value = value

    test = property(_getter, _setter)

    def __init__(self):
        self.test = 24

class UnicodeSlots(object):
    """Using unicode objects in __slots__ is okay.

    On Python 3.3 onward, u'' is equivalent to '',
    so this test should be safe for both versions.
    """
    __slots__ = (u'first', u'second')

    def __init__(self):
        self.first = 42
        self.second = 24


class DataDescriptor(object):
    def __init__(self, name, default=''):
        self.__name = name
        self.__default = default

    def __get__(self, inst, cls):
        return getattr(inst, self.__name, self.__default)

    def __set__(self, inst, value):
        setattr(inst, self.__name, value)


class NonDataDescriptor(object):
    def __get__(self, inst, cls):
        return 42


class SlotsWithDescriptor(object):
    __slots__ = ['_err']
    data_descriptor = DataDescriptor('_err')
    non_data_descriptor = NonDataDescriptor()
    missing_descriptor = Unknown()


def dont_emit_for_descriptors():
    inst = SlotsWithDescriptor()
    # This should not emit, because attr is
    # a data descriptor
    inst.data_descriptor = 'foo'
    inst.non_data_descriptor = 'lala' # [assigning-non-slot]


class ClassWithSlots(object):
    __slots__ = ['foobar']


class ClassReassigningDunderClass(object):
    __slots__ = ['foobar']

    def release(self):
        self.__class__ = ClassWithSlots


class ClassReassingingInvalidLayoutClass(object):
    __slots__ = []

    def release(self):
        self.__class__ = ClassWithSlots # [assigning-non-slot]


# pylint: disable=attribute-defined-outside-init
class ClassHavingUnknownAncestors(Unknown):
    __slots__ = ['yo']

    def test(self):
        self.not_yo = 42


# pylint: disable=wrong-import-order, wrong-import-position
from typing import (
    Generic,
    TypeVar,
)

TYPE = TypeVar('TYPE')


class Cls(Generic[TYPE]):
    """ Simple class with slots """
    __slots__ = ['value']

    def __init__(self, value):
        self.value = value



""" Test that import errors are detected. """
# pylint: disable=invalid-name, unused-import, no-absolute-import, bare-except, broad-except, wrong-import-order, wrong-import-position
import totally_missing # [import-error]

try:
    import maybe_missing
except ImportError:
    maybe_missing = None

try:
    import maybe_missing_1
except (ImportError, SyntaxError):
    maybe_missing_1 = None

try:
    import maybe_missing_2 # [import-error]
except ValueError:
    maybe_missing_2 = None


try:
    if maybe_missing:
        import really_missing
except ImportError:
    pass

# pylint: disable=no-name-in-module
from functional.s.syntax_error import toto # [syntax-error]



 # pylint: disable=no-self-use, no-method-argument, missing-docstring,


def function_3_args(first_argument, second_argument, third_argument):
    """three arguments function"""
    return first_argument, second_argument, third_argument


def function_default_arg(one=1, two=2):
    """fonction with default value"""
    return two, one


def args_out_of_order():
    """Tests for arguments-out-of-order"""
    first_argument = 1
    second_argument = 2
    third_argument = 3
    one = 1
    two = 2

    function_3_args(first_argument, third_argument, second_argument) # [arguments-out-of-order]
    function_3_args(second_argument, first_argument, # [arguments-out-of-order]
                    third_argument=third_argument)
    function_default_arg(two, one) # [arguments-out-of-order]

    # Checking exceptions:
    # supplying the same attribute twice instead of two different ones
    function_3_args(first_argument, first_argument, third_argument)

    # keyword passing
    function_default_arg(two=two, one=one)

    # anything other than named attributes
    function_3_args(1, list, 1 + 2)

    # The check assumes you know better if at least 1 arg does not match the function signature
    function_3_args(one, third_argument, second_argument)

    # we currently don't warn for the case where keyword args have been swapped
    function_default_arg(two=one, one=two)

    # ensure object methods are checked correctly despite implicit self
    class TestClass:
        @staticmethod
        def function_0_args():
            return
        def function_2_args(self, first_argument, second_argument=2):
            return first_argument, second_argument

    TestClass().function_2_args(second_argument, first_argument) # [arguments-out-of-order]
    TestClass().function_2_args(first_argument, second_argument=second_argument)
    TestClass.function_0_args()



# pylint: disable=missing-docstring,no-self-use,too-few-public-methods,wrong-import-position, useless-object-inheritance

REVISION = None

REVISION() # [not-callable]

def correct():
    return 1

REVISION = correct()

class Correct(object):
    """callable object"""

class MetaCorrect(object):
    """callable object"""
    def __call__(self):
        return self

INSTANCE = Correct()
CALLABLE_INSTANCE = MetaCorrect()
CORRECT = CALLABLE_INSTANCE()
INCORRECT = INSTANCE() # [not-callable]
LIST = []
INCORRECT = LIST() # [not-callable]
DICT = {}
INCORRECT = DICT() # [not-callable]
TUPLE = ()
INCORRECT = TUPLE() # [not-callable]
INT = 1
INCORRECT = INT() # [not-callable]

# Test calling properties. Pylint can detect when using only the
# getter, but it doesn't infer properly when having a getter
# and a setter.
class MyProperty(property):
    """ test subclasses """

class PropertyTest(object):
    """ class """

    def __init__(self):
        self.attr = 4

    @property
    def test(self):
        """ Get the attribute """
        return self.attr

    @test.setter
    def test(self, value):
        """ Set the attribute """
        self.attr = value

    @MyProperty
    def custom(self):
        """ Get the attribute """
        return self.attr

    @custom.setter
    def custom(self, value):
        """ Set the attribute """
        self.attr = value

PROP = PropertyTest()
PROP.test(40) # [not-callable]
PROP.custom() # [not-callable]

# Safe from not-callable when using properties.

class SafeProperty(object):
    @property
    def static(self):
        return staticmethod

    @property
    def klass(self):
        return classmethod

    @property
    def get_lambda(self):
        return lambda: None

    @property
    def other_function(self):
        def function(arg):
            return arg
        return function

    @property
    def dict_builtin(self):
        return dict

    @property
    def range_builtin(self):
        return range

    @property
    def instance(self):
        class Empty(object):
            def __call__(self):
                return 42
        return Empty()

    @property
    def does_not_make_sense(self):
        raise NotImplementedError

PROP1 = SafeProperty()
PROP1.static(2)
PROP1.klass(2)
PROP1.get_lambda()
PROP1.other_function(4)
PROP1.dict_builtin()
PROP1.range_builtin(4)
PROP1.instance()
PROP1.does_not_make_sense()


import missing # pylint: disable=import-error

class UnknownBaseCallable(missing.Blah):
    pass

UnknownBaseCallable()()



# pylint: disable=missing-docstring,invalid-name,undefined-variable,multiple-statements

# Variations of 'result'
result = ''
for number in ['1', '2', '3']:
    result += number  # [consider-using-join]

result = 'header'
for number in ['1', '2', '3']:
    result += number  # [consider-using-join]

result = another_result = ''
for number in ['1', '2', '3']:
    result += number  # [consider-using-join]

another_result = result = ''
for number in ['1', '2', '3']:
    result += number  # [consider-using-join]

result = 0  # result is not a string
for number in ['1', '2', '3']:
    result += number

RESULT = ''  # wrong name / initial variable missing
for number in ['1', '2', '3']:
    result += [number]

string_variable = ''
result = string_variable  # type of 'result' not obviously a string
for number in ['1', '2', '3']:
    result += number

result = ''
another_result = ''  # result defined too early
for number in ['1', '2', '3']:
    result += [number]

for number in ['1', '2', '3']:  # 'result'-definition missing
    result += number


# Variations of 'number'
result = ''  # no concatenation (iterator-name differs)
for name in ['1', '2', '3']:
    result += number

result = ''  # no concatenation (iterator-name differs)
for _ in ['1', '2', '3']:
    result += number
# 'exprlist' is not a single name
for index, number in ['1', '2', '3']:
    result += number


# Variations of 'iterable'
result = ''
for number in []:
    result += number  # [consider-using-join]

result = ''
for number in "a text":
    result += number  # [consider-using-join]

result = ''
for number in [1, 2, 3]:
    result += number  # [consider-using-join]

a_list = [1, 2, 3]
result = ''
for number in a_list:
    result += number  # [consider-using-join]

result = ''
for number in ['1', '2', '3']:
    result += number  # [consider-using-join]

result = ''
for number in undefined_iterable:
    result += number  # [consider-using-join]


# Variations of loop-body
result = ''  # addition is not the only part of the body
for number in ['1', '2', '3']:
    print(number)
    result += number

result = ''  # addition is not the only part of the body
for number in ['1', '2', '3']:
    result += number
    print(number)

result = ''  # augmented addition is not a simple one
for number in ['1', '2', '3']:
    result += '4' + number

result = ''  # assignment is not augmented
for number in ['1', '2', '3']:
    result = number

result = ''  # augmented assignment is not an addition
for number in ['1', '2', '3']:
    result -= number

result = ''  # addition is not the 'number'-iterable
for number in ['1', '2', '3']:
    result += another_number

result = ''
for number in ['1', '2', '3']: result += number  # [consider-using-join]

result = ''
for number in ['1']:
    result.result += number

# Does not emit if the body is more complex
result = {'context': 1}
result['context'] = 0
for number in ['1']:
    result1 = 42 + int(number)
    result['context'] += result1 * 24

# Does not crash if the previous sibling does not have AssignNames
result['context'] = 0
for number in ['1']:
    result['context'] += 24



"""All the things!"""
# pylint: disable=no-absolute-import
from .thing1 import THING1
from .thing2 import THING2
from .thing2 import THING1_PLUS_THING2
__revision__ = None

_ = (THING1, THING2, THING1_PLUS_THING2)
del _



# pylint: disable=missing-docstring
class Data:
    def __init__(self):
        self.shape = [None, 17]

    @property
    def ndim(self):
        return len(self.shape)

    def copy_move_axis(self, old_axis):
        if old_axis < 0:
            old_axis += self.ndim
            assert old_axis >= 0
        assert 0 <= old_axis < self.ndim

        new_shape = [None] * self.ndim
        self.shape = new_shape
        return self



from pylint.checkers import BaseChecker
from pylint.interfaces import IRawChecker


class MyRawChecker(BaseChecker):
    """check for line continuations with '\' instead of using triple
    quoted string or parenthesis
    """

    __implements__ = IRawChecker

    name = "custom_raw"
    msgs = {
        "W9901": (
            "use \\ for line continuation",
            "backslash-line-continuation",
            (
                "Used when a \\ is used for a line continuation instead"
                " of using triple quoted string or parenthesis."
            ),
        )
    }
    options = ()

    def process_module(self, node):
        """process a module

        the module's content is accessible via node.stream() function
        """
        with node.stream() as stream:
            for (lineno, line) in enumerate(stream):
                if line.rstrip().endswith("\\"):
                    self.add_message("backslash-line-continuation", line=lineno)


def register(linter):
    """required method to auto register this checker"""
    linter.register_checker(MyRawChecker(linter))



"""Test PEP 604 - Alternative Union syntax"""
# pylint: disable=missing-function-docstring,unused-argument,invalid-name,missing-class-docstring,inherit-non-class,too-few-public-methods
import dataclasses
import typing
from dataclasses import dataclass
from typing import NamedTuple, TypedDict


Alias = str | list[int]
lst = [typing.Dict[str, int] | None,]

var1: typing.Dict[str, int | None]
var2: int | str | None
var3: int | list[str | int]
var4: typing.Dict[typing.Tuple[int, int] | int, None]

cast_var = 1
cast_var = typing.cast(str | int, cast_var)

T = typing.TypeVar("T", int | str, bool)

(lambda x: 2)(int | str)

var: str | int

def func(arg: int | str):
    pass

def func2() -> int | str:
    pass

class CustomCls(int):
    pass

Alias2 = CustomCls |  str

var2 = CustomCls(1) | int(2)


# Check typing.NamedTuple
CustomNamedTuple = typing.NamedTuple(
    "CustomNamedTuple", [("my_var", int | str)])

class CustomNamedTuple2(NamedTuple):
    my_var: int | str

class CustomNamedTuple3(typing.NamedTuple):
    my_var: int | str


# Check typing.TypedDict
CustomTypedDict = TypedDict("CustomTypedDict", my_var=(int | str))

CustomTypedDict2 = TypedDict("CustomTypedDict2", {"my_var": int | str})

class CustomTypedDict3(TypedDict):
    my_var: int | str

class CustomTypedDict4(typing.TypedDict):
    my_var: int | str


# Check dataclasses
def my_decorator(*args, **kwargs):
    def wraps(*args, **kwargs):
        pass
    return wraps

@dataclass
class CustomDataClass:
    my_var: int | str

@dataclasses.dataclass
class CustomDataClass2:
    my_var: int | str

@dataclass()
class CustomDataClass3:
    my_var: int | str

@my_decorator
@dataclasses.dataclass
class CustomDataClass4:
    my_var: int | str



"""Checks use of consider-merging-isinstance"""
# pylint:disable=line-too-long, simplifiable-condition


def isinstances():
    "Examples of isinstances"
    var = range(10)

    # merged
    if isinstance(var[1], (int, float)):
        pass
    result = isinstance(var[2], (int, float))

    # not merged
    if isinstance(var[3], int) or isinstance(var[3], float) or isinstance(var[3], list) and True:  # [consider-merging-isinstance]
        pass
    result = isinstance(var[4], int) or isinstance(var[4], float) or isinstance(var[5], list) and False  # [consider-merging-isinstance]

    result = isinstance(var[5], int) or True or isinstance(var[5], float)  # [consider-merging-isinstance]

    infered_isinstance = isinstance
    result = infered_isinstance(var[6], int) or infered_isinstance(var[6], float) or infered_isinstance(var[6], list) and False   # [consider-merging-isinstance]
    result = isinstance(var[10], str) or isinstance(var[10], int) and var[8] * 14 or isinstance(var[10], float) and var[5] * 14.4 or isinstance(var[10], list)   # [consider-merging-isinstance]
    result = isinstance(var[11], int) or isinstance(var[11], int) or isinstance(var[11], float)   # [consider-merging-isinstance]

    result = isinstance(var[20])
    result = isinstance()

    # Combination merged and not merged
    result = isinstance(var[12], (int, float)) or isinstance(var[12], list)  # [consider-merging-isinstance]

    # not merged but valid
    result = isinstance(var[5], int) and var[5] * 14 or isinstance(var[5], float) and var[5] * 14.4
    result = isinstance(var[7], int) or not isinstance(var[7], float)
    result = isinstance(var[6], int) or isinstance(var[7], float)
    result = isinstance(var[6], int) or isinstance(var[7], int)
    return result



"""empty-comment test-case"""
A = 5  #
#
A = '#' + '1'
print(A)  #
print("A=", A)  # should not be an error#
A = "#pe\0ace#love#"  #
A = "peace#love"  # \0 peace'#'''' love#peace'''-'#love'-"peace#love"#
#######



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

from pylint.message import Message


def test_new_message(message_definitions):
    def build_message(message_definition, location_value):
        return Message(
            symbol=message_definition.symbol,
            msg_id=message_definition.msgid,
            location=[
                location_value["abspath"],
                location_value["path"],
                location_value["module"],
                location_value["obj"],
                location_value["line"],
                location_value["column"],
            ],
            msg=message_definition.msg,
            confidence="high",
        )

    template = "{path}:{line}:{column}: {msg_id}: {msg} ({symbol})"
    for message_definition in message_definitions:
        if message_definition.msgid == "E1234":
            e1234_message_definition = message_definition
        if message_definition.msgid == "W1234":
            w1234_message_definition = message_definition
    e1234_location_values = {
        "abspath": "1",
        "path": "2",
        "module": "3",
        "obj": "4",
        "line": "5",
        "column": "6",
    }
    w1234_location_values = {
        "abspath": "7",
        "path": "8",
        "module": "9",
        "obj": "10",
        "line": "11",
        "column": "12",
    }
    expected = (
        "2:5:6: E1234: Duplicate keyword argument %r in %s call (duplicate-keyword-arg)"
    )
    e1234 = build_message(e1234_message_definition, e1234_location_values)
    w1234 = build_message(w1234_message_definition, w1234_location_values)
    assert e1234.format(template) == expected
    assert w1234.format(template) == "8:11:12: W1234: message (msg-symbol)"



"""check builtin data descriptors such as mode and name attributes
on a file are correctly handled

bug notified by Pierre Rouleau on 2005-04-24
"""
from __future__ import print_function
__revision__ = None

class File(file):  # pylint: disable=file-builtin
    """ Testing new-style class inheritance from file"""

    #
    def __init__(self, name, mode="r", buffering=-1, verbose=False):
        """Constructor"""

        self.was_modified = False
        self.verbose = verbose
        super(File, self).__init__(name, mode, buffering)
        if self.verbose:
            print("File %s is opened.  The mode is: %s" % (self.name,
                                                           self.mode))

    #
    def write(self, a_string):
        """ Write a string to the file."""

        super(File, self).write(a_string)
        self.was_modified = True

    #
    def writelines(self, sequence):
        """ Write a sequence of strings to the file. """

        super(File, self).writelines(sequence)
        self.was_modified = True

    #
    def close(self):
        """Close the file."""

        if self.verbose:
            print("Closing file %s" % self.name)

        super(File, self).close()
        self.was_modified = False



"""Don't warn if the class is instantiated in its own body."""
# pylint: disable=missing-docstring, useless-object-inheritance


import abc


class Ala(object, metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def bala(self):
        pass

    @classmethod
    def portocala(cls):
        instance = cls()
        return instance



# pylint: disable=missing-docstring

def func(first, *, second): # [unused-argument, unused-argument]
    pass


def only_raises(first, second=42): # [unused-argument]
    if first == 24:
        raise ValueError



""" Test that superfluous else return are detected. """

# pylint:disable=invalid-name,missing-docstring,unused-variable


def foo1(x, y, z):
    for i in x:
        if i > y:  # [no-else-break]
            break
        else:
            a = z


def foo2(x, y, w, z):
    for i in x:
        if i > y:  # [no-else-break]
            break
        elif i > w:
            break
        else:
            a = z


def foo3(x, y, z):
    for i in x:
        if i > y:
            a = 1
            if z:  # [no-else-break]
                b = 2
                break
            else:
                c = 3
                break
        else:
            d = 4
            break


def foo4(x, y):
    for i in x:
        if i > x:  # [no-else-break]
            if y:
                a = 4
            else:
                b = 2
            break
        else:
            c = 3
        break


def foo5(x, y, z):
    for i in x:
        if i > y:  # [no-else-break]
            if y:
                a = 4
            else:
                b = 2
            break
        elif z:
            c = 2
        else:
            c = 3
        break


def foo6(x, y):
    for i in x:
        if i > x:
            if y:  # [no-else-break]
                a = 4
                break
            else:
                b = 2
        else:
            c = 3
        break


def bar1(x, y, z):
    for i in x:
        if i > y:
            break
        return z


def bar2(w, x, y, z):
    for i in w:
        if i > x:
            break
        if z < i:
            a = y
        else:
            break
        return


def bar3(x, y, z):
    for i in x:
        if i > y:
            if z:
                break
        else:
            return z
        return None


def bar4(x):
    for i in range(10):
        if x:  # [no-else-break]
            break
        else:
            try:
                return
            except ValueError:
                break


def nested1(x, y, z):
    for i in x:
        if i > x:
            for j in y:
                if j < z:
                    break
        else:
            a = z


def nested2(x, y, z):
    for i in x:
        if i > x:
            for j in y:
                if j > z:
                    break
                break
        else:
            a = z



#!/usr/bin/env python
# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Script used to generate the extensions file before building the actual documentation."""

import os
import re
import sys

import sphinx

from pylint.constants import MAIN_CHECKER_NAME
from pylint.lint import PyLinter
from pylint.utils import get_rst_title

# Some modules have been renamed and deprecated under their old names.
# Skip documenting these modules since:
# 1) They are deprecated, why document them moving forward?
# 2) We can't load the deprecated module and the newly renamed module at the
# same time without getting naming conflicts
DEPRECATED_MODULES = ["check_docs"]  # ==> docparams


def builder_inited(app):
    """Output full documentation in ReST format for all extension modules"""
    # PACKAGE/docs/exts/pylint_extensions.py --> PACKAGE/
    base_path = os.path.dirname(
        os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    )
    # PACKAGE/ --> PACKAGE/pylint/extensions
    ext_path = os.path.join(base_path, "pylint", "extensions")
    modules = []
    doc_files = {}
    for filename in os.listdir(ext_path):
        name, ext = os.path.splitext(filename)
        if name[0] == "_" or name in DEPRECATED_MODULES:
            continue
        if ext == ".py":
            modules.append("pylint.extensions.%s" % name)
        elif ext == ".rst":
            doc_files["pylint.extensions." + name] = os.path.join(ext_path, filename)
    modules.sort()
    if not modules:
        sys.exit("No Pylint extensions found?")

    linter = PyLinter()
    linter.load_plugin_modules(modules)

    extensions_doc = os.path.join(
        base_path, "doc", "technical_reference", "extensions.rst"
    )
    with open(extensions_doc, "w") as stream:
        stream.write(
            get_rst_title("Optional Pylint checkers in the extensions module", "=")
        )
        stream.write("Pylint provides the following optional plugins:\n\n")
        for module in modules:
            stream.write(f"- :ref:`{module}`\n")
        stream.write("\n")
        stream.write(
            "You can activate any or all of these extensions "
            "by adding a ``load-plugins`` line to the ``MASTER`` "
            "section of your ``.pylintrc``, for example::\n"
        )
        stream.write(
            "\n    load-plugins=pylint.extensions.docparams,"
            "pylint.extensions.docstyle\n\n"
        )
        by_checker = get_plugins_info(linter, doc_files)
        for checker, information in sorted(by_checker.items()):
            linter._print_checker_doc(information, stream=stream)


def get_plugins_info(linter, doc_files):
    by_checker = {}
    for checker in linter.get_checkers():
        if checker.name == MAIN_CHECKER_NAME:
            continue
        module = checker.__module__
        # Plugins only - skip over core checkers
        if re.match("pylint.checkers", module):
            continue
        # Find any .rst documentation associated with this plugin
        doc = ""
        doc_file = doc_files.get(module)
        if doc_file:
            with open(doc_file) as f:
                doc = f.read()
        try:
            by_checker[checker]["checker"] = checker
            by_checker[checker]["options"] += checker.options_and_values()
            by_checker[checker]["msgs"].update(checker.msgs)
            by_checker[checker]["reports"] += checker.reports
            by_checker[checker]["doc"] += doc
            by_checker[checker]["module"] += module
        except KeyError:
            by_checker[checker] = {
                "checker": checker,
                "options": list(checker.options_and_values()),
                "msgs": dict(checker.msgs),
                "reports": list(checker.reports),
                "doc": doc,
                "module": module,
            }
    return by_checker


def setup(app):
    app.connect("builder-inited", builder_inited)
    return {"version": sphinx.__display_version__}


if __name__ == "__main__":
    builder_inited(None)



# pylint: disable=missing-docstring, useless-object-inheritance
from __future__ import print_function


from enum import Enum


class Aaaa(object): # [too-few-public-methods]

    def __init__(self):
        pass

    def meth1(self):
        print(self)

    def _dontcount(self):
        print(self)


# Don't emit for these cases.
class Klass(object):
    """docstring"""

    def meth1(self):
        """first"""

    def meth2(self):
        """second"""


class EnoughPublicMethods(Klass):
    """We shouldn't emit too-few-public-methods for this."""


class BossMonster(Enum):
    """An enum does not need methods to be useful."""
    MEGASHARK = 1
    OCTOPUS = 2


class DumbList:
    """A class can define only special methods."""
    def __init__(self, iterable):
        self._list = list(iterable)

    def __len__(self):
        return len(self._list)

    def __getitem__(self, index):
        return self._list[index]



# pylint: disable=missing-module-docstring, undefined-variable
assert [foo, bar], "No AssertionError"
assert "There is an AssertionError" # [assert-on-string-literal]
assert "" # [assert-on-string-literal]



"""Checks import order rule with imports that isort could generate"""
# pylint: disable=unused-import
import astroid
import isort
from astroid import are_exclusive, decorators
from astroid.modutils import get_module_part, is_standard_module



import astroid

from pylint.checkers import BaseChecker
from pylint.interfaces import IAstroidChecker


# This is our checker class.
# Checkers should always inherit from `BaseChecker`.
class MyAstroidChecker(BaseChecker):
    """Add class member attributes to the class locals dictionary."""

    # This class variable defines the type of checker that we are implementing.
    # In this case, we are implementing an AST checker.
    __implements__ = IAstroidChecker

    # The name defines a custom section of the config for this checker.
    name = "custom"
    # The priority indicates the order that pylint will run the checkers.
    priority = -1
    # This class variable declares the messages (ie the warnings and errors)
    # that the checker can emit.
    msgs = {
        # Each message has a code, a message that the user will see,
        # a unique symbol that identifies the message,
        # and a detailed help message
        # that will be included in the documentation.
        "W0001": ("Message that will be emitted", "message-symbol", "Message help")
    }
    # This class variable declares the options
    # that are configurable by the user.
    options = (
        # Each option definition has a name which is used on the command line
        # and in config files, and a dictionary of arguments
        # (similar to those to those to
        # argparse.ArgumentParser.add_argument).
        (
            "store-locals-indicator",
            {
                "default": "properties",
                "help": (
                    "The expression name that indicates that the locals should "
                    "be stored"
                ),
            },
        ),
    )

    def visit_call(self, node):
        """Called when a :class:`.astroid.node_classes.Call` node is visited.

        See :mod:`astroid` for the description of available nodes.

        :param node: The node to check.
        :type node: astroid.node_classes.Call
        """
        if not (
            isinstance(node.func, astroid.Attribute)
            and isinstance(node.func.expr, astroid.Name)
            and node.func.expr.name == self.config.store_locals_indicator
            and node.func.attrname == "create"
        ):
            return
        in_class = node.frame()
        for param in node.args:
            in_class.locals[param.name] = node


def register(linter):
    """This required method auto registers the checker.

    :param linter: The linter to register the checker to.
    :type linter: pylint.lint.PyLinter
    """
    linter.register_checker(MyAstroidChecker(linter))



# pylint: disable=missing-docstring, invalid-name
# https://github.com/PyCQA/pylint/issues/3165

list([])
tuple([])

list([0 for y in list(range(10))]) # [consider-using-generator]
tuple([0 for y in list(range(10))]) # [consider-using-generator]

list(0 for y in list(range(10)))
tuple(0 for y in list(range(10)))



"""Warning about assigning self/cls variable."""
from __future__ import print_function
# pylint: disable=too-few-public-methods, useless-object-inheritance

class Foo(object):
    """Class with methods that check for self/cls assignment"""

    # pylint: disable=no-self-argument,no-self-use
    def self_foo(bar_):
        """Instance method, should warn for bar"""
        bar_ = 10  # [self-cls-assignment]

    # pylint: disable=no-self-use
    def self_foofoo(self, lala):
        """Instance method, should warn for self"""
        self = lala  # [self-cls-assignment]

    @classmethod
    def cls_foo(cls):
        """Class method, should warn for cls"""
        cls = 'tada'  # [self-cls-assignment]

    # pylint: disable=unused-argument
    @staticmethod
    def static_foo(lala):
        """Static method, no warnings"""
        lala = 10


# pylint: disable=unused-argument
def free_foo(bar_, lala):
    """Free function, no warnings"""
    bar_ = lala


class TestNonLocal:
    """Test class for nonlocal assignment of self"""

    def function(self, param):
        """This function uses nonlocal to reassign self"""

        def _set_param(param):
            nonlocal self
            self = param  # [self-cls-assignment]

        _set_param(param)
        return self



# pylint: disable=missing-docstring
import dataclasses
import typing
from dataclasses import dataclass


@dataclasses.dataclass
class ScheduledTxSearchModel:
    date_from = None
    date_to = None


@dataclass
class ScheduledTxSearchModelOne:
    date = None


@dataclass(frozen=True)
class Test:
    some_integer: int


class Example(typing.NamedTuple):
    some_int: int


@dataclasses.dataclass(frozen=True)
class Point:
    """A three dimensional point with x, y and z components."""

    attr1: float
    attr2: float
    attr3: float

    def to_array(self):
        """Convert to a NumPy array `np.array((x, y, z))`."""
        return self.attr1



# pylint: disable=missing-docstring,too-few-public-methods, useless-object-inheritance
'''Test that a function is considered a method when looked up through a class.'''

class Clazz(object):
    'test class'

    def __init__(self, value):
        self.value = value

def func(arg1, arg2):
    'function that will be used as a method'
    return arg1.value + arg2

Clazz.method = func

VAR = Clazz(1).method(2)



"""Fixture for testing missing documentation in docparams."""


def _private_func(param1):
    if param1:
        raise Exception('Example')
    return param1


def _private_func2(param1):
    yield param1



# pylint: disable=missing-docstring, broad-except, unreachable, try-except-raise, raise-missing-from
# pylint: disable=unused-variable, too-few-public-methods, invalid-name, useless-object-inheritance

try:
    raise # [misplaced-bare-raise]
except Exception:
    pass

try:
    pass
except Exception:
    raise

# pylint: disable=misplaced-comparison-constant
try:
    pass
except Exception:
    if 1 == 2:
        raise

def test():
    try:
        pass
    except Exception:
        def chest():
            try:
                pass
            except Exception:
                raise
        raise

def test1():
    try:
        if 1 > 2:
            def best():
                raise # [misplaced-bare-raise]
    except Exception:
        pass
    raise # [misplaced-bare-raise]
raise # [misplaced-bare-raise]

try:
    pass
finally:
    # This might work or might not to, depending if there's
    # an error raised inside the try block. But relying on this
    # behaviour can be error prone, so we decided to warn
    # against it.
    raise # [misplaced-bare-raise]


class A(object):
    try:
        pass
    except Exception:
        raise
    raise # [misplaced-bare-raise]

# This works in Python 2, but the intent is nevertheless
# unclear. It will also not work on Python 3, so it's best
# not to rely on it.
exc = None
try:
    1/0
except ZeroDivisionError as exc:
    pass
if exc:
    raise # [misplaced-bare-raise]

# Don't emit if we're in ``__exit__``.
class ContextManager(object):
    def __enter__(self):
        return self
    def __exit__(self, *args):
        raise


def test_dont_trigger_in_finally_clause_found_in_exception_handler():
    try:
        raise ValueError('bad value')
    except ValueError:
        try:
            raise IOError('failed')
        finally:
            raise



""" Functional tests for method deprecation. """
# pylint: disable=missing-docstring, super-init-not-called, not-callable
import base64
import cgi
import inspect
import logging
import nntplib
import platform
import unittest
import xml.etree.ElementTree


class MyTest(unittest.TestCase):
    def test(self):
        self.assert_(True)  # [deprecated-method]

xml.etree.ElementTree.Element('tag').getchildren()  # [deprecated-method]
xml.etree.ElementTree.Element('tag').getiterator()  # [deprecated-method]
xml.etree.ElementTree.XMLParser('tag', None, None).doctype(None, None, None)  # [deprecated-method]
nntplib.NNTP(None).xpath(None) # [deprecated-method]


inspect.getargspec(None) # [deprecated-method]
logging.warn("a") # [deprecated-method]
platform.popen([]) # [deprecated-method]
base64.encodestring("42") # [deprecated-method]
base64.decodestring("42") # [deprecated-method]
cgi.escape("a") # [deprecated-method]


class SuperCrash(unittest.TestCase):

    def __init__(self):
        # should not crash.
        super()()

xml.etree.ElementTree.iterparse(None)


class Tests(unittest.TestCase):

    def test_foo(self):
        self.assertEquals(2 + 2, 4)  # [deprecated-method]
        self.assertNotEquals(2 + 2, 4)  # [deprecated-method]
        self.assertAlmostEquals(2 + 2, 4)  # [deprecated-method]
        self.assertNotAlmostEquals(2 + 2, 4)  # [deprecated-method]
        self.assert_("abc" == "2")  # [deprecated-method]

        self.assertRaisesRegex(ValueError, "exception")
        self.assertRegex("something", r".+")



# pylint: disable=R0903, useless-object-inheritance
"""
Simple test case for an annoying behavior in pylint.
"""

__revision__ = 'pouet'

class Test(object):
    """Smallest test case for reported issue."""

    def __init__(self):
        self._thing = None

    @property
    def myattr(self):
        """Getter for myattr"""
        return self._thing

    @myattr.setter
    def myattr(self, value):
        """Setter for myattr."""
        self._thing = value

Test().myattr = 'grou'



"""Test that `abc.ABCMeta.__new__` does not trigger too-many-function-arguments when referred

https://github.com/PyCQA/pylint/issues/2335
"""
# pylint: disable=missing-class-docstring,unused-argument,arguments-differ
from abc import ABCMeta


class NodeCheckMetaClass(ABCMeta):
    def __new__(cls, name, bases, namespace, **kwargs):
        return ABCMeta.__new__(cls, name, bases, namespace)



import astroid

from pylint.checkers import BaseChecker, DeprecatedMixin, utils
from pylint.interfaces import UNDEFINED, IAstroidChecker
from pylint.testutils import CheckerTestCase, Message


class _DeprecatedChecker(DeprecatedMixin, BaseChecker):
    __implements__ = (IAstroidChecker,)
    name = "deprecated"

    msgs = {
        "W1505": (
            "Using deprecated method %s()",
            "deprecated-method",
            "The method is marked as deprecated and will be removed in "
            "a future version of Python. Consider looking for an "
            "alternative in the documentation.",
        )
    }

    @utils.check_messages(
        "deprecated-method",
    )
    def visit_call(self, node):
        """Visit a Call node."""
        try:
            for inferred in node.func.infer():
                self.check_deprecated_method(node, inferred)
        except astroid.InferenceError:
            return

    def deprecated_methods(self):
        return {"deprecated_func", ".Deprecated.deprecated_method"}


class TestDeprecatedChecker(CheckerTestCase):
    CHECKER_CLASS = _DeprecatedChecker

    def test_deprecated_function(self):
        # Tests detecting deprecated function
        node = astroid.extract_node(
            """
        def deprecated_func():
            pass

        deprecated_func()
        """
        )
        with self.assertAddsMessages(
            Message(
                msg_id="deprecated-method",
                args=("deprecated_func",),
                node=node,
                confidence=UNDEFINED,
            )
        ):
            self.checker.visit_call(node)

    def test_deprecated_method(self):
        # Tests detecting deprecated method
        node = astroid.extract_node(
            """
        class Deprecated:
            def deprecated_method():
                pass

        d = Deprecated()
        d.deprecated_method()
        """
        )
        with self.assertAddsMessages(
            Message(
                msg_id="deprecated-method",
                args=("deprecated_method",),
                node=node,
                confidence=UNDEFINED,
            )
        ):
            self.checker.visit_call(node)

    def test_no_message(self):
        # Tests not raising error when no deprecated functions/methods are present.
        node = astroid.extract_node(
            """
        class MyClass:
            def mymethod():
                pass

        MyClass().mymethod()

        def myfunc():
            pass

        myfunc()
        """
        )
        with self.assertNoMessages():
            self.checker.visit_call(node)



from dataclasses import dataclass


@dataclass
class InventoryItem:
    """Class for keeping track of an item in inventory."""

    name: str
    unit_price: float
    quantity_on_hand: int = 0

    def total_cost(self) -> float:
        return self.unit_price * self.quantity_on_hand



# pylint: disable=missing-docstring, superfluous-parens


try:
    1/0
except (ValueError | TypeError): # [wrong-exception-operation]
    pass

try:
    1/0
except (ValueError + TypeError): # [wrong-exception-operation]
    pass


try:
    1/0
except (ValueError < TypeError): # [wrong-exception-operation]
    pass



# pylint: disable=too-few-public-methods,print-statement, useless-object-inheritance,missing-docstring
"""check method hidding ancestor attribute
"""
from __future__ import print_function

class Abcd(object):
    """dummy"""
    def __init__(self):
        self.abcd = 1

class Cdef(Abcd):
    """dummy"""
    def abcd(self): # [method-hidden]
        """test
        """
        print(self)

class AbcdMixin:
    def abcd(self):
        pass

class Dabc(AbcdMixin, Abcd):
    def abcd(self):
        pass

class CustomProperty:
    """dummy"""
    def __init__(self, _):
        pass

    def __get__(self, obj, __):
        if not obj:
            return self
        return 5

    def __set__(self, _, __):
        pass

class Ddef:
    """dummy"""
    def __init__(self):
        self.five = "five"

    @CustomProperty
    def five(self):
        """Always 5."""
        return self


def my_decorator(*args, **kwargs):
    return CustomProperty(*args, **kwargs)


class Foo:
    def __init__(self):
        self._bar = 42
        self._baz = 84

    @my_decorator
    def method(self):  # E0202
        return self._baz

    @method.setter
    def method(self, value):
        self._baz = value

    def do_something_with_baz(self, value):
        self.method = value


class One:
    def __init__(self, one=None):
        if one is not None:
            self.one = one

    def one(self): # [method-hidden]
        pass

class Two(One):
    def one(self):
        pass

try:
    import unknown as js
except ImportError:
    import json as js


class JsonEncoder(js.JSONEncoder):
    # pylint: disable=useless-super-delegation,super-with-arguments
    def default(self, o):
        return super(JsonEncoder, self).default(o)



"""https://www.logilab.org/ticket/8771"""

from __future__ import print_function

def generator():
    """yield as assignment"""
    yield 45
    xxxx = yield 123
    print(xxxx)

def generator_fp1(seq):
    """W0631 false positive"""
    for val in seq:
        pass
    for val in seq:
        yield val

def generator_fp2():
    """E0601 false positive"""
    xxxx = 12
    yield xxxx



# Copyright (c) 2013-2014 Google, Inc.
# Copyright (c) 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
# Copyright (c) 2015-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2015 Aru Sahni <arusahni@gmail.com>
# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
# Copyright (c) 2017 Pierre Sassoulas <pierre.sassoulas@cea.fr>
# Copyright (c) 2017-2018, 2020 Anthony Sottile <asottile@umich.edu>
# Copyright (c) 2017 ttenhoeve-aa <ttenhoeve@appannie.com>
# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
# Copyright (c) 2018-2019, 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
# Copyright (c) 2018 Pierre Sassoulas <pierre.sassoulas@wisebim.fr>
# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

import io
import re

from pylint.utils import utils


def test__basename_in_blacklist_re_match():
    patterns = [re.compile(".*enchilada.*"), re.compile("unittest_.*")]
    assert utils._basename_in_blacklist_re("unittest_utils.py", patterns)
    assert utils._basename_in_blacklist_re("cheese_enchiladas.xml", patterns)


def test__basename_in_blacklist_re_nomatch():
    patterns = [re.compile(".*enchilada.*"), re.compile("unittest_.*")]
    assert not utils._basename_in_blacklist_re("test_utils.py", patterns)
    assert not utils._basename_in_blacklist_re("enchilad.py", patterns)


def test_decoding_stream_unknown_encoding():
    """decoding_stream should fall back to *some* decoding when given an
    unknown encoding.
    """
    binary_io = io.BytesIO(b"foo\nbar")
    stream = utils.decoding_stream(binary_io, "garbage-encoding")
    # should still act like a StreamReader
    ret = stream.readlines()
    assert ret == ["foo\n", "bar"]


def test_decoding_stream_known_encoding():
    binary_io = io.BytesIO("€".encode("cp1252"))
    stream = utils.decoding_stream(binary_io, "cp1252")
    assert stream.read() == "€"



# Copyright (c) 2006-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
# Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
# Copyright (c) 2014 Google, Inc.
# Copyright (c) 2014 Arun Persaud <arun@nubati.net>
# Copyright (c) 2015-2020 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
# Copyright (c) 2016-2017 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2018 Reverb C <reverbc@users.noreply.github.com>
# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
# Copyright (c) 2020 Damien Baty <damien.baty@polyconseil.fr>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""non regression tests for pylint, which requires a too specific configuration
to be incorporated in the automatic functional test framework
"""
# pylint: disable=redefined-outer-name

import os
import sys
from os.path import abspath, dirname, join

import astroid
import pytest

from pylint import testutils

REGR_DATA = join(dirname(abspath(__file__)), "regrtest_data")
sys.path.insert(1, REGR_DATA)


@pytest.fixture(scope="module")
def reporter():
    return testutils.GenericTestReporter


@pytest.fixture(scope="module")
def disable():
    return ["I"]


@pytest.fixture
def finalize_linter(linter):
    """call reporter.finalize() to cleanup
    pending messages if a test finished badly
    """
    yield linter
    linter.reporter.finalize()


def Equals(expected):
    return lambda got: got == expected


@pytest.mark.parametrize(
    "file_name, check",
    [
        ("package.__init__", Equals("")),
        ("precedence_test", Equals("")),
        ("import_package_subpackage_module", Equals("")),
        ("pylint.checkers.__init__", lambda x: "__path__" not in x),
        (join(REGR_DATA, "classdoc_usage.py"), Equals("")),
        (join(REGR_DATA, "module_global.py"), Equals("")),
        (join(REGR_DATA, "decimal_inference.py"), Equals("")),
        (join(REGR_DATA, "absimp", "string.py"), Equals("")),
        (join(REGR_DATA, "bad_package"), lambda x: "Unused import missing" in x),
    ],
)
def test_package(finalize_linter, file_name, check):
    finalize_linter.check(file_name)
    got = finalize_linter.reporter.finalize().strip()
    assert check(got)


@pytest.mark.parametrize(
    "file_name",
    [
        join(REGR_DATA, "import_assign.py"),
        join(REGR_DATA, "special_attr_scope_lookup_crash.py"),
        join(REGR_DATA, "try_finally_disable_msg_crash"),
    ],
)
def test_crash(finalize_linter, file_name):
    finalize_linter.check(file_name)


@pytest.mark.parametrize(
    "fname", [x for x in os.listdir(REGR_DATA) if x.endswith("_crash.py")]
)
def test_descriptor_crash(fname, finalize_linter):
    finalize_linter.check(join(REGR_DATA, fname))
    finalize_linter.reporter.finalize().strip()


@pytest.fixture
def modify_path():
    cwd = os.getcwd()
    sys.path.insert(0, "")
    yield
    sys.path.pop(0)
    os.chdir(cwd)


@pytest.mark.usefixtures("modify_path")
def test_check_package___init__(finalize_linter):
    filename = "package.__init__"
    finalize_linter.check(filename)
    checked = list(finalize_linter.stats["by_module"].keys())
    assert checked == [filename]

    os.chdir(join(REGR_DATA, "package"))
    finalize_linter.check("__init__")
    checked = list(finalize_linter.stats["by_module"].keys())
    assert checked == ["__init__"]


def test_pylint_config_attr():
    mod = astroid.MANAGER.ast_from_module_name("pylint.lint.pylinter")
    pylinter = mod["PyLinter"]
    expect = [
        "OptionsManagerMixIn",
        "object",
        "MessagesHandlerMixIn",
        "ReportsHandlerMixIn",
        "BaseTokenChecker",
        "BaseChecker",
        "OptionsProviderMixIn",
    ]
    assert [c.name for c in pylinter.ancestors()] == expect
    assert list(astroid.Instance(pylinter).getattr("config"))
    inferred = list(astroid.Instance(pylinter).igetattr("config"))
    assert len(inferred) == 1
    assert inferred[0].root().name == "optparse"
    assert inferred[0].name == "Values"
